// ====================================================================
// 
//   ECM: The Extensible Container Model
//   Copyright (C) 2004 THALES
//   Contact: openccm-ecm@objectweb.org
//
//   This library is free software; you can redistribute it and/or
//   modify it under the terms of the GNU Lesser General Public
//   License as published by the Free Software Foundation; either
//   version 2.1 of the License, or any later version.
//
//   This library is distributed in the hope that it will be useful,
//   but WITHOUT ANY WARRANTY; without even the implied warranty of
//   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
//   Lesser General Public License for more details.
//
//   You should have received a copy of the GNU Lesser General Public
//   License along with this library; if not, write to the Free Software
//   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
//   USA
//
//   Initial developer(s): Mathieu Vadet.
//   Initial Funding: IST COACH European project (IST-2001-34445)
//                    http://www.ist-coach.org 
//
// ====================================================================



package org.objectweb.ccm.sessiondemo;

import org.objectweb.corba.runtime.*;

/**
 **  Demo2 application deployment.
 **/
public class SessionDemo
implements Runnable, java.awt.event.ActionListener
{
    // main
    private ORBService _orb_service;
    private ClientHome _client_home;
    private ServerHome  _server_home;

    public
    SessionDemo(ORBService orbs)
    {
        // main
        _orb_service = orbs;
        _client_home = null;
        _server_home = null;
    }

    //
    //  java.lang.Runnable
    //

    final public void
    run()
    { 
        try {
            // need to register value factory for Components::Cookie
            ValueFactory vfact = new ValueFactoryImpl(org.omg.Components.CookieHelper.id(), new CookieFactoryImpl());
            _orb_service.register_valuefactory(vfact);

            // obtain the INS scheme of the registration service
            RegistrationService regs = org.objectweb.corba.runtime.Runtime.getRegistrationService();
            INSRegistrationScheme scheme = (INSRegistrationScheme)regs.get_scheme(INSRegistrationScheme.SCHEME_ID);

            // obtain component homes
            System.err.println("## Obtain homes");
            org.omg.CORBA.Object obj = null;
            obj = scheme.lookup("ClientHome", _orb_service);
            _client_home = ClientHomeHelper.narrow(obj);
            obj = scheme.lookup("ServerHome", _orb_service);
            _server_home = ServerHomeHelper.narrow(obj);

            // create GUI for component creation
            javax.swing.JPanel panel = new javax.swing.JPanel(new java.awt.BorderLayout());
            javax.swing.JButton newb = new javax.swing.JButton("new client");
            newb.setActionCommand("new");
            newb.addActionListener(this);
            panel.add(newb, java.awt.BorderLayout.NORTH);

            javax.swing.JFrame frame = new javax.swing.JFrame("Client factory");
            frame.getContentPane().add(panel);
            frame.pack();
            frame.show();

        } catch (Exception ex) {
            ex.printStackTrace();
            throw new Error(ex.getMessage());
        }

        System.out.println("Again ....");
    }

    // 
    //  java.awt.event.ActionListener
    //

    final public void
    actionPerformed(java.awt.event.ActionEvent e)
    {
        String cmd = e.getActionCommand();

        try {
            Client cl1 = _client_home.create();
            cl1.connect_server_home(_server_home);
            cl1.configuration_complete();
        } catch (Exception ex) {
            ex.printStackTrace();
            throw new Error(ex.getMessage());
        }

    }

    static public Runnable
    main(ORBService orbs)
    {
        return new SessionDemo(orbs);
    }
}







