// ====================================================================
// 
//   ECM: The Extensible Container Model
//   Copyright (C) 2004 THALES
//   Contact: openccm-ecm@objectweb.org
//
//   This library is free software; you can redistribute it and/or
//   modify it under the terms of the GNU Lesser General Public
//   License as published by the Free Software Foundation; either
//   version 2.1 of the License, or any later version.
//
//   This library is distributed in the hope that it will be useful,
//   but WITHOUT ANY WARRANTY; without even the implied warranty of
//   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
//   Lesser General Public License for more details.
//
//   You should have received a copy of the GNU Lesser General Public
//   License along with this library; if not, write to the Free Software
//   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
//   USA
//
//   Initial developer(s): Mathieu Vadet.
//   Initial Funding: IST COACH European project (IST-2001-34445)
//                    http://www.ist-coach.org 
//
// ====================================================================



package org.objectweb.ccm.sessiondemo;

/**
 **  Implementation of the OMG IDL3 <tt>Server</tt> component type.
 **/
public class ServerImpl
extends Server_MainSegBase
implements java.io.Serializable
{
    // server
    static private Product[] _aproducts = {
        new Product("A", 10),
        new Product("B", 20),
        new Product("C", 30),
        new Product("D", 10),
        new Product("E", 50),
        new Product("F", 90),
        new Product("G", 20),
        new Product("H", 60),
        new Product("I", 10),
        new Product("J", 70),
    };

    static private java.util.Hashtable _products;

    static {
        _products = new java.util.Hashtable();
        for (int i=0;i<_aproducts.length;i++) {
            _products.put(_aproducts[i].id, _aproducts[i]);
        }
    }

    transient private java.util.ArrayList _purchased;
    private String _spurchased;

    // default constructor
    public
    ServerImpl()
    {
        // server
        _purchased = null;
        _spurchased = null;
    }

    //
    //  overriden operations
    //

    final public void
    ccm_activate()
    throws org.omg.Components.CCMException
    {
        System.err.println("### [ServerImpl] ###");
        System.err.println("### [ServerImpl] ccm_activate called");
        System.err.println("### [ServerImpl] ###");

        // NOTE: first activation, create array list
        if (_purchased==null) {
            _purchased = new java.util.ArrayList();
        }

        // NOTE: re-populate array list from saved data
        if (_spurchased!=null) {
            // break the string
            String purchased = _spurchased;
            int idx = purchased.indexOf(',');
            String pid = null;
            while (idx!=-1) {
                pid = purchased.substring(0, idx);
                purchased = purchased.substring(idx+1);
                idx = purchased.indexOf(',');
                _purchased.add(pid);
            }

            _purchased.add(purchased);
            _spurchased = null;
        }
    }

    final public void
    ccm_passivate()
    throws org.omg.Components.CCMException
    {
        System.err.println("### [ServerImpl] ###");
        System.err.println("### [ServerImpl] ccm_passivate called");
        System.err.println("### [ServerImpl] ###");

        // NOTE: store purchased items ids in a string
        StringBuffer buffer = new StringBuffer();
        String[] purchased = (String[])_purchased.toArray(new String[0]);
        for (int i=0;i<purchased.length;i++) {
            buffer.append(purchased[i]+",");
        }

        // 
        _spurchased = buffer.toString();
    }

    final public void
    ccm_remove()
    throws org.omg.Components.CCMException
    {
        System.err.println("### [ServerImpl] ###");
        System.err.println("### [ServerImpl] ccm_remove called");
        System.err.println("### [ServerImpl] ###");

        _purchased.clear();
        _purchased = null;
        _spurchased = null;
    }

    //
    //  IDL:objectweb.org/ccm/sessiondemo/Server:1.0
    //

    //
    //  IDL:objectweb.org/ccm/sessiondemo/IServer:1.0
    //

    final public Product[]
    list_products()
    {
        return _aproducts;
    }

    final public void
    purchase_products(String[] pids)
    {
        _purchased.addAll(java.util.Arrays.asList(pids));
    }

    final public void
    remove_products(String[] pids)
    {
        for (int i=0;i<pids.length;i++) {
            for (int j=0;j<_purchased.size();j++) {
                if (pids[i].equals(_purchased.get(j))) {
                    _purchased.remove(j);
                    break;
                }
            }
        }
    }

    final public Product[]
    view_caddie()
    {
        Product[] purchased = new Product[_purchased.size()];
        for (int i=0;i<purchased.length;i++) {
            purchased[i] = (Product)_products.get(_purchased.get(i));
        }

        return purchased;
    }
}
