// ====================================================================
// 
//   ECM: The Extensible Container Model
//   Copyright (C) 2004 THALES
//   Contact: openccm-ecm@objectweb.org
//
//   This library is free software; you can redistribute it and/or
//   modify it under the terms of the GNU Lesser General Public
//   License as published by the Free Software Foundation; either
//   version 2.1 of the License, or any later version.
//
//   This library is distributed in the hope that it will be useful,
//   but WITHOUT ANY WARRANTY; without even the implied warranty of
//   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
//   Lesser General Public License for more details.
//
//   You should have received a copy of the GNU Lesser General Public
//   License along with this library; if not, write to the Free Software
//   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
//   USA
//
//   Initial developer(s): Mathieu Vadet.
//   Initial Funding: IST COACH European project (IST-2001-34445)
//                    http://www.ist-coach.org 
//
// ====================================================================



package org.objectweb.ccm.logdemo;

/**
 **  This is the ECM-unaware (and CCM standard) implementation of the OMG IDL3
 **  <tt>AsyncServer</tt> component type.
 **/
public class AsyncServerImpl
extends AsyncServer_MainSegBase
{
    // server
    private String _name;
    private javax.swing.JFrame _frame;
    private javax.swing.JTextArea _text;

    public
    AsyncServerImpl()
    {
        // server
        _name = null;
        _frame = null;
        _text = null;
    }

    //
    //  overriden operations
    //

    final public void
    configuration_complete()
    throws org.omg.Components.InvalidConfiguration
    {
        // create a GUI
        // CENTER: text field to display client inputs
        _text = new javax.swing.JTextArea(40, 20);
        _text.setEditable(false);

        // MAIN panel
        javax.swing.JPanel panel = new javax.swing.JPanel(new java.awt.BorderLayout());
        panel.add(new javax.swing.JScrollPane(_text), java.awt.BorderLayout.CENTER);
        _frame = new javax.swing.JFrame(_name + " GUI");
        _frame.setSize(400, 300);
        _frame.getContentPane().add(panel);
        _frame.pack();
        _frame.show();
    }

    //
    //  IDL:omg.org/Components/SessionComponent:1.0
    //

    final public void
    ccm_activate()
    throws org.omg.Components.CCMException
    {
        // NOTE: TODO: should be done externally using the CSD info
        // register value factory
        getCCM2Context().register_valuefactory(AsyncMessageHelper.id(), new AsyncMessageFactoryImpl());

        // show the frame if hidden
        if ((_frame!=null) && (!_frame.isShowing())){
            _frame.show();
        }
    }

    final public void
    ccm_passivate()
    throws org.omg.Components.CCMException
    {
        // hide the frame and clear the text field
        _text.setText(null);
        _frame.hide();
    }

    final public void
    ccm_remove()
    throws org.omg.Components.CCMException
    {
        // Release the associated frame.
        _frame.dispose();
        _frame = null;
    }

    //
    //  IDL:org.omg/Components/EventConsumerBase:1.0
    //

    final public void
    push_event(org.omg.Components.EventBase evt)
    throws org.omg.Components.BadEventType
    {
        if (evt instanceof AsyncMessage) {
            push_AsyncMessage((AsyncMessage)evt);
            return ;
        }

        throw new org.omg.Components.BadEventType();
    }

    //
    //  IDL:objectweb.org/ccm/logdemo/MessageConsumer:1.0
    //

    final public void
    push_AsyncMessage(AsyncMessage amsg)
    {
        // add content to the text field
        _text.append("["+amsg.msg.header+"] "+amsg.msg.content + "\n");
    }

    //
    //  IDL:objectweb.org/ccm/logdemo/AsyncServer:1.0
    //

    final public void
    name(String n)
    {
        _name = n;

        if (_frame!=null) {
            _frame.setTitle(_name+" GUI");
        }
    }

    final public String
    name()
    {
        return _name;
    }
}
