@echo off
REM ==================================================================== 
REM
REM   ECM: The Extensible Container Model
REM   Copyright (C) 2004 THALES
REM   Contact: openccm-ecm@objectweb.org
REM
REM   This library is free software; you can redistribute it and/or
REM   modify it under the terms of the GNU Lesser General Public
REM   License as published by the Free Software Foundation; either
REM   version 2.1 of the License, or any later version.
REM
REM   This library is distributed in the hope that it will be useful,
REM   but WITHOUT ANY WARRANTY; without even the implied warranty of
REM   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
REM   Lesser General Public License for more details.
REM
REM   You should have received a copy of the GNU Lesser General Public
REM   License along with this library; if not, write to the Free Software
REM   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
REM   USA
REM
REM   Initial developer(s): Mathieu Vadet
REM   Initial funding: IST COACH European Project (IST-2001-34445)
REM                    http://www.ist-coach.org
REM
REM ==================================================================== 


REM
REM  Starts the demo
REM

setlocal

goto begin

:usage
    echo usage: logdemo_start.bat [options]
    echo Starts the demo.
    echo --
    echo The following options are supported:
    echo   /h, /help    Print this message
    echo   /debug       Activate debug mode (optional)
    echo   /withlog     Activate log service (optional)
    exit /B 1


:begin
set debug=
set withlog=

:parse_args
if not "%1"=="" (
    if "%1"=="/h" (
        goto usage
    ) else if "%1"=="/help" (
        goto usage
    ) else if "%1"=="/debug" (
        set debug=/debug
    ) else if "%1"=="/withlog" (
        set withlog=t
    )

    shift
    goto parse_args
)

set lszip=
set suffix=

if defined withlog (
    echo Starting demo WITH log service
    set lszip=DCE:LogService=file:@ECM.install.zip.dir@\ccm_logservice.zip
    set suffix=WithLog
) else (
    echo Starting demo WITHOUT log service
)

REM start the NS
call @ECM.install.bin.dir@\ns_start.bat %debug%

REM start the ExtCI with component zips
set client_zip=DCE:Client%suffix%Impl=file:@ECM.install.zip.dir@\logdemo_client.zip
set server_zip=DCE:Server%suffix%Impl=file:@ECM.install.zip.dir@\logdemo_server.zip
set asyncserver_zip=DCE:AsyncServer%suffix%Impl=file:@ECM.install.zip.dir@\logdemo_asyncserver.zip

call @ECM.install.bin.dir@\extci_start.bat %debug% /autoload "%lszip%,%client_zip%,%server_zip%,%asyncserver_zip%"

REM start the LogService if enabled
set lsref=
set services=
if defined withlog (
    call @ECM.install.bin.dir@\logserver_start.bat %debug% LogServer

    REM obtain logservice IOR and set it as an initial ref
    call @ECM.install.bin.dir@\logserver_ior.bat %debug% /url LogServer > %TEMP%\ls_ior.tmp
    for /f %%i in (%TEMP%\ls_ior.tmp) do (
        set args=-ORBInitRef LogService=%%i %args%
    )
    del %TEMP%\ls_ior.tmp

    REM services
    set services=/services "DCE:LogService"
)

REM start a CCM runtime with the client home
echo Starting CCM runtime with the client home
call @ECM.install.bin.dir@\ccm_runtime_start.bat %debug% /args "%args%" /uuid DCE:Client%suffix%Impl %services% ClientHome

REM start a CCM runtime with the server home
echo Starting CCM runtime with the server home
call @ECM.install.bin.dir@\ccm_runtime_start.bat %debug% /args "%args%" /uuid DCE:Server%suffix%Impl %services% ServerHome

REM start a CCM runtime with the async server home
echo Starting CCM runtime with the async server home
call @ECM.install.bin.dir@\ccm_runtime_start.bat %debug% /args "%args%" /uuid DCE:AsyncServer%suffix%Impl %services% AsyncServerHome


REM start the main
set entrypt=org.objectweb.ccm.logdemo.LogDemo.main
set location=@ECM.install.lib.dir@\logdemo_main.jar
call @ECM.install.bin.dir@\corba_runtime_start.bat %debug% /entrypoint %entrypt% /location %location% LogDemoMain

echo Demo started

:eof