// ====================================================================
// 
//   ECM: The Extensible Container Model
//   Copyright (C) 2004 THALES
//   Contact: openccm-ecm@objectweb.org
//
//   This library is free software; you can redistribute it and/or
//   modify it under the terms of the GNU Lesser General Public
//   License as published by the Free Software Foundation; either
//   version 2.1 of the License, or any later version.
//
//   This library is distributed in the hope that it will be useful,
//   but WITHOUT ANY WARRANTY; without even the implied warranty of
//   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
//   Lesser General Public License for more details.
//
//   You should have received a copy of the GNU Lesser General Public
//   License along with this library; if not, write to the Free Software
//   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
//   USA
//
//   Initial developer(s): Mathieu Vadet.
//   Initial Funding: IST COACH European project (IST-2001-34445)
//                    http://www.ist-coach.org 
//
// ====================================================================



package org.objectweb.ccm.runtime;

import org.objectweb.corba.runtime.*;

/**
 **  <p></p>
 **/
public class ComponentArchiveImpl
extends ArchiveServantBase
implements Servant, org.coach.ECM.ComponentArchiveOperations
{
    // system component
    static final private String _class_name = "ComponentArchiveImpl";

    // servant
    static final private String _type_id =  org.coach.ECM.ComponentArchiveHelper.id();

    // archive
    private ORBService _orb_service;
    private String _uuid;
    private String _archive_location;
    private String _dtd_location;

    // component archive
    private ExtComponentInstallationImpl _extci;
    private String _code_location;
    private String _code_entrypoint;
    private String _csd_location;
    private String _ccd_location;
    private String _home_repid;
    private String _component_repid;
    private org.coach.ECM.ValuetypeFactoryDependency[] _valuefactories;
    private java.util.HashMap _sname2policies;

    public
    ComponentArchiveImpl(ExtComponentInstallationImpl extci,
                         ORBService orbs,
                         String dtdloc,
                         String uuid,
                         String loc)
    {
        super(uuid, loc);

        // archive
        _orb_service = orbs;
        _uuid = uuid;
        _archive_location = loc;
        _dtd_location = dtdloc;

        // component archive
        _extci = extci;
        _code_location = null;
        _code_entrypoint = null;
        _csd_location = null;
        _ccd_location = null;
        _home_repid = null;
        _component_repid = null;
        _valuefactories = new org.coach.ECM.ValuetypeFactoryDependency[0];
        _sname2policies = new java.util.HashMap();
    }

    //
    //  abstract operations implementation
    //

    final protected ORBService
    getORBService()
    {
        return _orb_service;
    }

    //
    //  internal operations
    //

    private void 
    initializeInternalFields()
    {
        final String opname = "initializeInternalFields";
        // open zip
        java.util.zip.ZipFile zip = openZip();

        // extract CSD location
        java.util.Enumeration entries = zip.entries();
        java.util.zip.ZipEntry entry = null;
        java.util.zip.ZipEntry csdentry  = null;
        while (entries.hasMoreElements()) {
            entry = (java.util.zip.ZipEntry)entries.nextElement();
            if (entry.getName().endsWith(".csd")) {
                csdentry = entry;
            }
        }

        // check if CSD found
        if (csdentry==null) {
            // NOTE: TODO: should be reported as an exception
            final String msg = "FAILED (no CSD found in "+_uuid+")";
            TheLogger.error(_class_name, opname, msg);
            return ;
        }

        // copy CSD file locally
        String csdfname = extractFile(zip, csdentry);
        _csd_location = csdfname;

        // parse CSD file
        CSDParser csdparser = CSDParser.parse(_csd_location, _dtd_location, _uuid);

        // obtain code entry point
        _code_entrypoint = csdparser.code_entrypoint;

        // obtain valuetype factories
        _valuefactories = (org.coach.ECM.ValuetypeFactoryDependency[])csdparser.valuetype_factories.toArray(new org.coach.ECM.ValuetypeFactoryDependency[0]);

        // extract zip entry using result as entry name
        entries = zip.entries();
        java.util.zip.ZipEntry ccdentry  = null;
        java.util.zip.ZipEntry codelocentry  = null;
        String ccdname = csdparser.descriptor_location;
        String codeloc = csdparser.code_location;
        while (entries.hasMoreElements()) {
            entry = (java.util.zip.ZipEntry)entries.nextElement();
            if (entry.getName().equals(ccdname)) {
                ccdentry = entry;
            }
            else if (entry.getName().equals(codeloc)) {
                codelocentry = entry;
            }
        }

        // check if code archive found
        if (codelocentry==null) {
            // NOTE: TODO: should be reported as an exception
            final String msg = "FAILED (no code library found in "+_uuid+")";
            TheLogger.error(_class_name, opname, msg);
            return ;
        }

        // copy code archive locally
        String codelocfname = extractFile(zip, codelocentry);
        _code_location = codelocfname;

        // check if CCD found
        if (ccdentry==null) {
            // NOTE: TODO: should be reported as an exception
            final String msg = "FAILED (CCD file not found in "+_uuid+")";
            TheLogger.error(_class_name, opname, msg);
            return ;
        }

        // copy CCD file locally
        String ccdfname = extractFile(zip, ccdentry);
        _ccd_location = ccdfname;

        // parse CCD
        CCDParser ccdparser = CCDParser.parse(_ccd_location, _dtd_location, _extci);
        _home_repid = ccdparser.home_repid;
        _component_repid = ccdparser.component_repid;
        _sname2policies = ccdparser.sname2policies;

        // close CSD
        // NOTE: how ?
        // close CCSD
        // NOTE: how ?

        // close zip
        try {
            zip.close();
        } catch(java.io.IOException ex) {
            // NOTE: TODO: should be reported as an exception
            // ignore for the moment
            TheLogger.error(_class_name, opname, "IGNORE", ex);
        }
    }

    private void
    updatePolicyList(java.util.ArrayList list, org.coach.ECM.CCDPolicyRef[] pols)
    {
        // NOTE: doesn't add already existing policies
        for (int i=0;i<pols.length;i++) {
            // check if a policy of the same service is already defined
            org.coach.ECM.CCDPolicyRef pol = null;
            String sid = pols[i].service_id;
            boolean found = false;
            for (int j=0;j<list.size();j++) {
                pol = (org.coach.ECM.CCDPolicyRef)list.get(j);
                if (sid.equals(pol.service_id)) {
                    found = true;
                    break;
                }

            }

            if (!found) {
                // add policy to the list
                list.add(pols[i]);
            }
        }
    }

    //
    //  IDL:org/objectweb/corba/runtime/SystemComponent:1.0
    //

    final public void
    system_configuration_complete(SystemConfiguration cfg)
    {
        // initiliaze some internal fields
        initializeInternalFields();
    }

    final public void
    destroy()
    {
        // TODO
    }

    //
    //  IDL:org/objectweb/corba/runtime/Servant:1.0
    //

    final public String
    type_id()
    {
        return _type_id;
    }

    final public org.omg.PortableServer.Servant
    as_native_servant()
    {
        return new org.coach.ECM.ComponentArchivePOATie(this);
    }

    //
    //  IDL:org/objectweb/ccm/runtime/Archive:1.0
    //

    // public String get_fileinarchive_location(String name, String link)
    // public String get_href_location(String href)
    // public String get_link_location(String link)
    // public String get_ins_as_objref(String ins)
    // public String uuid()

    //
    //  IDL:coach.org/ECM/ComponentArchive:1.0
    //

    final public String 
    csd_location()
    {
        return _csd_location;
    }

    final public String
    ccd_location()
    {
        return _ccd_location;
    }

    final public String
    code_location()
    {
        return _code_location;
    }

    final public String
    code_entrypoint()
    {
        return _code_entrypoint;
    }

    final public String
    home_repid()
    {
        return _home_repid;
    }

    final public String
    component_repid()
    {
        return _component_repid;
    }

    final public org.coach.ECM.ValuetypeFactoryDependency[]
    get_valuetype_factories()
    {
        return _valuefactories;
    }

    final public org.coach.ECM.CCDPolicyRef[]
    get_policies_for_target(String sname)
    {
        final String opname = "get_policies_for_target";
        String msg = "scoped name: "+sname;
        TheLogger.debug(_class_name, opname, msg);

        // NOTE: wildcard management: for a scoped name ::a::b, we look for:
        //  - ::a::b
        //  - ::a::*
        // NOTE: attribute management: for a getter (or setter) attribute ::I::_get_attr, we look for:
        //  - ::I::_get_attr
        //  - ::I::_get_*
        //  - ::I::attr
        //  - ::I::*

        // NOTE2: duplicated policies are removed (ie policies of the same service)
        // the precedence order is: ::a::b > ::a::* 
        //                          ::I::_get_attr > ::I::attr > ::I::_get_* > ::I::*

        java.util.ArrayList res = new java.util.ArrayList();
        java.util.ArrayList starres = new java.util.ArrayList();

        org.coach.ECM.CCDPolicyRef[] prefs = null;
        int idx = sname.lastIndexOf(':');
        // name==b or name==_get_attr
        String name = sname.substring(idx+1, sname.length());
        // pname==::a or name==::I
        String pname = sname.substring(0, idx-1);

        // normal case
        //msg = "normal case: "+sname;
        //TheLogger.debug(_class_name, "get_policies_for_target", msg);
        prefs = (org.coach.ECM.CCDPolicyRef[])_sname2policies.get(sname);
        if (prefs!=null) {
            res.addAll(java.util.Arrays.asList(prefs));
        }

        // starname==::a::* or starname==::I::*
        String starname = pname+"::*";

        //msg = "star case: "+starname;
        //TheLogger.debug(_class_name, "get_policies_for_target", msg);

        prefs = (org.coach.ECM.CCDPolicyRef[])_sname2policies.get(starname);
        if (prefs!=null) {
            starres.addAll(java.util.Arrays.asList(prefs));
        }

        // check if attribute
        // quick check to avoid string comparison in most cases
        if (name.charAt(0)=='_') {
            if (name.startsWith("_get_")) {
                // attrname==::I::attr
                String attrname = pname+"::"+name.substring(5, name.length());

                //msg = "attribute case: "+attrname;
                //TheLogger.debug(_class_name, "get_policies_for_target", msg);

                prefs = (org.coach.ECM.CCDPolicyRef[])_sname2policies.get(attrname);
                if (prefs!=null) {
                    updatePolicyList(res, prefs);
                }

                // attrstarname==::I::_get_*
                String attrstarname = pname+"::_get_*";

                //msg = "attribute star case: "+attrstarname;
                //TheLogger.debug(_class_name, "get_policies_for_target", msg);

                prefs = (org.coach.ECM.CCDPolicyRef[])_sname2policies.get(attrstarname);
                if (prefs!=null) {
                    updatePolicyList(starres, prefs);
                }
            }
            else if (name.startsWith("_set_")) {
                // attrname==::I::attr
                String attrname = pname+"::"+name.substring(5, name.length());

                //msg = "attribute case: "+attrname;
                //TheLogger.debug(_class_name, "get_policies_for_target", msg);

                prefs = (org.coach.ECM.CCDPolicyRef[])_sname2policies.get(attrname);
                if (prefs!=null) {
                    updatePolicyList(res, prefs);
                }

                // attrstarname==::I::_set_*
                String attrstarname = pname+"::_set_*";

                //msg = "attribute star case: "+attrstarname;
                //TheLogger.debug(_class_name, "get_policies_for_target", msg);

                prefs = (org.coach.ECM.CCDPolicyRef[])_sname2policies.get(attrstarname);
                if (prefs!=null) {
                    updatePolicyList(starres, prefs);
                }
            }
        }

        // update normal list with star list
        org.coach.ECM.CCDPolicyRef[] starprefs = (org.coach.ECM.CCDPolicyRef[])starres.toArray(new org.coach.ECM.CCDPolicyRef[0]);
        updatePolicyList(res, starprefs);

        prefs = (org.coach.ECM.CCDPolicyRef[])res.toArray(new org.coach.ECM.CCDPolicyRef[0]);

        if (prefs.length==0) {
            TheLogger.debug(_class_name, opname, "no policy found");
        } else {
            msg = "#"+prefs.length+" policies found";
            TheLogger.debug(_class_name, opname, msg);
            for (int i=0;i<prefs.length;i++) {
                msg = "#"+i+": "+prefs[i].service_id+" / "+prefs[i].policyref;
                TheLogger.debug(_class_name, opname, msg);
            }
        }

        return prefs;
    }

    final public String
    exec_xpath_request_in_csd(String req)
    {
        final String opname = "exec_xpath_request_in_csd";
        final String msg = "FAILED (not implemented, rethrown)";
        TheLogger.debug(_class_name, opname, msg);

        throw new org.omg.CORBA.NO_IMPLEMENT();
    }

    final public String
    exec_xpath_request_in_ccd(String req)
    {
        final String opname = "exec_xpath_request_in_ccd";
        final String msg = "FAILED (not implemented, rethrown)";
        TheLogger.debug(_class_name, opname, msg);

        throw new org.omg.CORBA.NO_IMPLEMENT();
    }
}
