// ====================================================================
// 
//   ECM: The Extensible Container Model
//   Copyright (C) 2004 THALES
//   Contact: openccm-ecm@objectweb.org
//
//   This library is free software; you can redistribute it and/or
//   modify it under the terms of the GNU Lesser General Public
//   License as published by the Free Software Foundation; either
//   version 2.1 of the License, or any later version.
//
//   This library is distributed in the hope that it will be useful,
//   but WITHOUT ANY WARRANTY; without even the implied warranty of
//   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
//   Lesser General Public License for more details.
//
//   You should have received a copy of the GNU Lesser General Public
//   License along with this library; if not, write to the Free Software
//   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
//   USA
//
//   Initial developer(s): Mathieu Vadet.
//   Initial Funding: IST COACH European project (IST-2001-34445)
//                    http://www.ist-coach.org 
//
// ====================================================================



package org.objectweb.ccm.runtime;

import org.objectweb.corba.runtime.*;

/**
 **  <p></p>
 **/
public class ComponentRuntimeMain
{
    static final private String _class_name = "ComponentRuntimeMain";

    //
    //  public operations
    //

    static public ComponentRuntimeConfiguration
    parseConfiguration(String[] args)
    {
        final String opname = "parseConfiguration";
        ComponentRuntimeConfiguration config = new ComponentRuntimeConfiguration();

        // parse command line
        String uuid = null;
        String suuids = null;
        String rid = null;
        String riorfile = null;
        java.util.Vector virefs = new java.util.Vector();

        for (int i=0;i<args.length;i++) {
            if (args[i].equals("-ComponentUUID")) {
                uuid=  args[i+1];
                i++;
            }

            if (args[i].equals("-ServiceUUIDs")) {
                suuids=  args[i+1];
                i++;
            }

            if (args[i].equals("-RuntimeId")) {
                rid = args[i+1];
                i++;
                // NOTE: set it as a property also as it's used in the TheLogger class
                java.lang.System.getProperties().setProperty("runtime.id", rid);
            }

            if (args[i].equals("-RuntimeIORFile")) {
                riorfile = args[i+1];
                i++;
            }

            if (args[i].equals("-ORBInitRef")) {
                virefs.add(args[i+1]);
                i++;
            }

        }
    
        // parse initial references
        String[] irefs = (String[])virefs.toArray(new String[0]);
        org.coach.ECM.DegeneratedServiceRef[] dsrefs = new org.coach.ECM.DegeneratedServiceRef[irefs.length];
        StringifiedInitialReference[] sirefs = new StringifiedInitialReference[irefs.length];

        int idx = 0;
        for (int i=0;i<irefs.length;i++) {
            // normally, an initial reference has the following format:
            //   "<reg_name>=<ref_ior>"

            String msg = "initial reference found: "+irefs[i];
            TheLogger.debug(_class_name, opname, msg);

            // look for the "="
            idx = irefs[i].indexOf("=");
            dsrefs[i] = new org.coach.ECM.DegeneratedServiceRef();
            dsrefs[i].corba_type = "unknown";
            dsrefs[i].orb_registration_name = irefs[i].substring(0, idx);
            dsrefs[i].initial_reference = irefs[i].substring(idx+1);

            sirefs[i] = new StringifiedInitialReference();
            sirefs[i].name = dsrefs[i].orb_registration_name;
            sirefs[i].ref = dsrefs[i].initial_reference;
        }

        // store
        config.implementation_uuid = uuid;
        config.runtime_id = rid;
        config.runtime_iorfile = riorfile;
        config.dservice_refs = dsrefs;
        config.initial_references = sirefs;

        // check if some service uuids are set
        if (suuids==null) {
            return config;
        }

        // break the string
        java.util.Vector vuuids = new java.util.Vector();
        idx = suuids.indexOf(',');
        String pattern = "";
        while (idx!=-1) {
            pattern = suuids.substring(0, idx);
            if (!pattern.equals("")) {
                vuuids.add(pattern);
            }

            suuids = suuids.substring(idx+1);
            idx = suuids.indexOf(',');
        }

        vuuids.add(suuids);

        // store
        config.service_uuids = (String[])vuuids.toArray(new String[0]);
        return config;
    }

    //
    //  main
    //

    static public void
    main(String[] args)
    {
        final String opname = "main";
        // log a start message
        TheLogger.log(_class_name, opname, "Loading home");

        // parse config
        ComponentRuntimeConfiguration config = parseConfiguration(args);

        // load servant
        ComponentRuntime.loadHome(config);

        // log a end message
        TheLogger.log(_class_name, opname, "Home loaded");

        System.err.println("Launcher waiting for this usefull message ...");
        System.out.println("... or for this one ? :o(");
    }
}
