// ====================================================================
// 
//   ECM: The Extensible Container Model
//   Copyright (C) 2004 THALES
//   Contact: openccm-ecm@objectweb.org
//
//   This library is free software; you can redistribute it and/or
//   modify it under the terms of the GNU Lesser General Public
//   License as published by the Free Software Foundation; either
//   version 2.1 of the License, or any later version.
//
//   This library is distributed in the hope that it will be useful,
//   but WITHOUT ANY WARRANTY; without even the implied warranty of
//   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
//   Lesser General Public License for more details.
//
//   You should have received a copy of the GNU Lesser General Public
//   License along with this library; if not, write to the Free Software
//   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
//   USA
//
//   Initial developer(s): Mathieu Vadet.
//   Initial Funding: IST COACH European project (IST-2001-34445)
//                    http://www.ist-coach.org 
//
// ====================================================================



package org.objectweb.ccm.runtime;

import org.objectweb.corba.runtime.*;

/**
 **  <p>Codec for monolithic components object ids.</p>
 **/
public class MonolithicIdCodec
{
    // codec
    static final private String _class_name = "MonolithicIdCodec";
    private ORBService _orb_service;
    private org.omg.IOP.Codec _iiop12codec;

    // default constructor
    public
    MonolithicIdCodec(ORBService orbs)
    {
        // codec
        _orb_service = orbs;

        // create IIOP 1.2 codec
        org.omg.CORBA.Object obj = _orb_service.resolve_initial_references("CodecFactory");
        org.omg.IOP.CodecFactory fact = org.omg.IOP.CodecFactoryHelper.narrow(obj);
        org.omg.IOP.Encoding enc12 = new org.omg.IOP.Encoding();
        enc12.format = org.omg.IOP.ENCODING_CDR_ENCAPS.value;
        enc12.major_version = (byte)1;
        enc12.minor_version = (byte)2;

        try {
            _iiop12codec = fact.create_codec(enc12);
        } catch (org.omg.IOP.CodecFactoryPackage.UnknownEncoding ex) {
            final String opname = "SessionIdCodec";
            TheLogger.error(_class_name, opname, "FAILED (IIOP 1.2 no supported)", ex);
        }
    }

    //
    //  public operations
    //

    final public byte[]
    encode(MonolithicId sid)
    {
        final String opname = "encode";

        // marshall
        org.omg.CORBA.Any any_id = _orb_service.create_any();
        MonolithicIdHelper.insert(any_id, sid);
        byte[] id = null;
        try {
            id = _iiop12codec.encode_value(any_id);
        } catch (org.omg.IOP.CodecPackage.InvalidTypeForEncoding ex) {
            // NOTE: TODO: in which case can this exception occur ?
            TheLogger.error(_class_name, opname, "FAILED (invalid type)", ex);
            return null;
        } catch (Throwable ex) {
            TheLogger.debug(_class_name, opname, "FAILED", ex);
            return null;
        }

        return id;
    }

    final public MonolithicId
    decode(byte[] id)
    {
        final String opname = "decode";

        // unmarshall
        org.omg.CORBA.Any any_id = _orb_service.create_any();
        try {
            any_id = _iiop12codec.decode_value(id, MonolithicIdHelper.type());
        } catch (org.omg.IOP.CodecPackage.FormatMismatch ex) {
            // if bad format, do not fail
            TheLogger.debug(_class_name, opname, "IGNORE", ex);
            return null;
        } catch (org.omg.IOP.CodecPackage.TypeMismatch ex) {
            // if bad type, fail (should not happen)
            TheLogger.error(_class_name, opname, "FAILED (wrong type)", ex);
            return null;
        } catch (Throwable ex) {
            TheLogger.debug(_class_name, opname, "FAILED", ex);
            return null;
        }

        // extract and build component id
        MonolithicId mid = MonolithicIdHelper.extract(any_id);

        return mid;
    }
}
