#! /bin/sh
# ==================================================================== 
#
#   ECM: The Extensible Container Model
#   Copyright (C) 2004 THALES
#   Contact: openccm-ecm@objectweb.org
#
#   This library is free software; you can redistribute it and/or
#   modify it under the terms of the GNU Lesser General Public
#   License as published by the Free Software Foundation; either
#   version 2.1 of the License, or any later version.
#
#   This library is distributed in the hope that it will be useful,
#   but WITHOUT ANY WARRANTY; without even the implied warranty of
#   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
#   Lesser General Public License for more details.
#
#   You should have received a copy of the GNU Lesser General Public
#   License along with this library; if not, write to the Free Software
#   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
#   USA
#
#   Initial developer(s): Mathieu Vadet
#   Initial funding: IST COACH European Project (IST-2001-34445)
#                    http://www.ist-coach.org
#
# ==================================================================== 


#
#  Starts a component server
#

usage() {
    echo "usage: cs_start.sh [options]"
    echo "Starts a CCM ComponentServer."
    echo "--"
    echo "The following options are supported:"
    echo "  -h, --help     Print this message"
    echo "  -d, --debug    Activate debug mode (optional)"
    echo "  --ns nsior     NS to use for registrations, default to NS_at_hostname NS (optional)"
    echo "  name           The logical name of the CS, no default value (mandatory)"
    exit 1;
}

# parse arguments
debug=
name=
ns=

while [ -n "$1" ] ;
do
    if [ "$1" = "-h" ] || [ "$1" = "--help" ];
    then
        usage ;
    elif [ "$1" = "-d" ] || [ "$1" = "--debug" ] ;
    then
        debug=--debug ;
    elif [ "$1" = "--ns" ] ;
    then
        ns=--ns $2
        shift ;
    else
        name=$1 ;
    fi ;

    shift ;
done

# check if name is set
if [ -z "$name" ] ;
then
    echo "ERROR: logical name of the CCM ComponentServer is missing."
    exit 1;
fi

REM obtain ExtCI IOR and set it as an initial ref
extci_ior=`@ECM.install.bin.dir@/extci_ior.sh $debug --url` 
args=-ORBInitRef ExtComponentInstallation=%extci_ior %args

# set code location and entrypt
location=@ECM.install.lib.dir@/ccm_runtime_cs.jar
entrypt=org.objectweb.ccm.runtime.ComponentServerImpl.create_servant

# delegate to corba_runtime_start script
@ECM.install.bin.dir@/corba_runtime_start.sh --args "$args" --location $location --entrypoint $entrypt $ns $debug $name
