#! /bin/sh
# ==================================================================== 
#
#   ECM: The Extensible Container Model
#   Copyright (C) 2004 THALES
#   Contact: openccm-ecm@objectweb.org
#
#   This library is free software; you can redistribute it and/or
#   modify it under the terms of the GNU Lesser General Public
#   License as published by the Free Software Foundation; either
#   version 2.1 of the License, or any later version.
#
#   This library is distributed in the hope that it will be useful,
#   but WITHOUT ANY WARRANTY; without even the implied warranty of
#   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
#   Lesser General Public License for more details.
#
#   You should have received a copy of the GNU Lesser General Public
#   License along with this library; if not, write to the Free Software
#   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
#   USA
#
#   Initial developer(s): Mathieu Vadet
#   Initial funding: IST COACH European Project (IST-2001-34445)
#                    http://www.ist-coach.org
#
# ==================================================================== 


#
#  Starts an Extended Component Installation object
#

usage() {
    echo "usage: extci_start.sh [options]"
    echo "Starts a CCM ExtendedComponentInstallation."
    echo "--"
    echo "The following options are supported:"
    echo "  -h, --help                         Print this message"
    echo "  -d, --debug                        Activate debug mode (optional)"
    echo "  --ns nsior                         NS to use for registrations, default to NS_at_hostname NS (optional)"
    echo "  --autoload \"uuid=loc,uuid=loc*\"    List of component archives (separated by commas) to be loaded (optional)"
    echo "  name                               The logical name of the ExtCI, ExtCI_at_hostname by default (optional)"
    exit 1 ;
}

# parse arguments
autoload=
debug=
name=
ns=
args=
jvmargs=

while [ -n "$1" ] ;
do
    if [ "$1" = "-h" ] || [ "$1" = "--help" ];
    then
        usage ;
    elif [ "$1" = "-d" ] || [ "$1" = "--debug" ] ;
    then
        debug=--debug ;
    elif [ "$1" = "--ns" ] ;
    then
        ns=--ns $2
        shift ;
    elif [ "$1" = "--autoload" ] ;
    then
        autoload=$2
        shift ;
    else
        name=$1 ;
    fi ;

    shift ;
done

# check if name is set
if [ -z "$name" ] ;
then
    if [ -n "$debug" ] ;
    then
        echo "WARNING: Will use default name for CCM ExtendedComponentInstallation."
    fi ;

    hostname=`hostname`
    name=ExtCI_at_$hostname ;
fi

# create and set zip cache
zip_cache=$CCM_RUNTIME_REPOSITORY/$name
mkdir $zip_cache
jvmargs="-Dzip.cache=$zip_cache $jvmargs"
args="-ZipCache $zip_cache $args"

# set DTD location
jvmargs="-Ddtd.location=@ECM.install.dtd.dir@ $jvmargs"
args="-DTDLocation=@ECM.install.dtd.dir@ $args"

if [ -n "$autoload" ] ;
then
    jvmargs="-Darchives.autoload=$autoload $jvmargs"
    args="-ArchivesAutoload $autoload $args"
fi

# set code location and entrypt
location=@ECM.install.lib.dir@/ccm_runtime_extci.jar
entrypt=org.objectweb.ccm.runtime.ExtComponentInstallationImpl.create_servant

# start
if [ -n "$debug" ] ;
then
    echo "INFO: Starting CCM ExtCI $name ..."
fi

@ECM.install.bin.dir@/corba_runtime_start.sh --jvmargs "$jvmargs" --args "$args" --location $location --entrypoint $entrypt $ns $debug $name

if [ -n "$debug" ] ;
then
    echo "INFO: The CCM ExtCI $name is started."
fi

