#! /bin/sh
# ==================================================================== 
#
#   ECM: The Extensible Container Model
#   Copyright (C) 2004 THALES
#   Contact: openccm-ecm@objectweb.org
#
#   This library is free software; you can redistribute it and/or
#   modify it under the terms of the GNU Lesser General Public
#   License as published by the Free Software Foundation; either
#   version 2.1 of the License, or any later version.
#
#   This library is distributed in the hope that it will be useful,
#   but WITHOUT ANY WARRANTY; without even the implied warranty of
#   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
#   Lesser General Public License for more details.
#
#   You should have received a copy of the GNU Lesser General Public
#   License along with this library; if not, write to the Free Software
#   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
#   USA
#
#   Initial developer(s): Mathieu Vadet
#   Initial funding: IST COACH European Project (IST-2001-34445)
#                    http://www.ist-coach.org
#
# ==================================================================== 


#
#  Starts a server activator
#

usage() {
    echo "usage: sa_start.sh [options]"
    echo "Starts a CCM ServerActivator."
    echo "It also starts local CORBA CosNaming service and a CCM ExtendedComponentInstallation."
    echo "--"
    echo "The following options are supported:"
    echo "  -h, --help     Print this message"
    echo "  -d, --debug    Activate debug mode (optional)"
    echo "  --ns nsior     NS to use for registrations, default to NS_at_hostname NS (optional)"
    echo "  name           The logical name of the SA, SA_at_hostname by default (optional)"
    echo "                 NOTE: if name is set, then the ExtCI will use ExtCI_at_name as logical name, ExtCI_at_hostname otherwise,"
    echo "                       and the NS will use NS_at_name as logical name, NS_at_hostname otherwise"
    exit 1 ;
}

# parse arguments
debug=
name=
extci_name=
ns_name=
ns=
args=
jvmargs=

while [ -n "$1" ] ;
do
    if [ "$1" = "-h" ] || [ "$1" = "--help" ];
    then
        usage ;
    elif [ "$1" = "-d" ] || [ "$1" = "--debug" ] ;
    then
        debug=--debug ;
    elif [ "$1" = "--ns" ] ;
        ns=--ns $2
        shift ;
    then
    else
        name=$1
        extci_name=ExtCI_at_$1
        ns_name=NS_at_$1
    fi ;

    shift ;
done

# check if name is set
if [ -z "$name" ] ;
then
    if [ -n "$debug" ] ;
    then
        echo "WARNING: Will use default name for CCM ServerActivator."
    fi ;

    hostname=`hostname`
    name=SA_at_$hostname ;
fi


# start the NS
@ECM.install.bin.dir@/ns_start.sh $debug $ns_name

# obtain local NS IOR
ns_ior=`@ECM.install.bin.dir@/ns_ior.sh $debug --url $ns_name`

# add reference of local NS
args=-ORBInitRef LocalNameService=$ns_ior

if [ -z "$ns" ] ;
then
    # use local NS as remote NS
    ns=--ns $ns_ior
fi

# start an extended component installation
@ECM.install.bin.dir@/extci_start.sh $debug $ns $extci_name

# obtain ExtCI IOR and set it as an initial ref
extci_ior=`@ECM.install.bin.dir@/extci_ior.sh $debug --url $extci_name`

# add reference of ExtCI
args=-ORBInitRef ExtComponentInstallation=$extci_ior $args

# set "os.option.char" java property
jvmargs=-Dos.option.str=/

#
jvmargs=-DECM.install.lib.dir=@ECM.install.lib.dir@ $jvmargs
jvmargs=-DECM.install.launcher.dir=@ECM.install.launcher.dir@ $jvmargs

# set code location and entrypt
location=@ECM.install.lib.dir@/ccm_runtime_sa.jar
entrypt=org.objectweb.ccm.runtime.ServerActivatorImpl.create_servant

# delegate to corba_runtime_start script
@ECM.install.bin.dir@/corba_runtime_start.sh --jvmargs "$jvmargs" --args "$args" --location $location --entrypoint $entrypt $ns $debug $name
