#! /bin/sh
# ==================================================================== 
#
#   ECM: The Extensible Container Model
#   Copyright (C) 2004 THALES
#   Contact: openccm-ecm@objectweb.org
#
#   This library is free software; you can redistribute it and/or
#   modify it under the terms of the GNU Lesser General Public
#   License as published by the Free Software Foundation; either
#   version 2.1 of the License, or any later version.
#
#   This library is distributed in the hope that it will be useful,
#   but WITHOUT ANY WARRANTY; without even the implied warranty of
#   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
#   Lesser General Public License for more details.
#
#   You should have received a copy of the GNU Lesser General Public
#   License along with this library; if not, write to the Free Software
#   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
#   USA
#
#   Initial developer(s): Mathieu Vadet
#   Initial funding: IST COACH European Project (IST-2001-34445)
#                    http://www.ist-coach.org
#
# ==================================================================== 


#
#  Stops a server activator
#

usage() {
    echo "usage: sa_stop.sh [options]"
    echo "Stops a CCM ServerActivator."
    echo "--"
    echo "The following options are supported:"
    echo "  -h, --help     Print this message"
    echo "  -d, --debug    Activate debug mode (optional)"
    echo "  name           The logical name of the SA, SA_at_hostname by default (optional)"
    exit 1 ;
}

# parse arguments
name=
extci_name=
ns_name=
debug=

while [ -n "$1" ] ;
do
    if [ "$1" = "-h" ] || [ "$1" = "--help" ];
    then
        usage ;
    elif [ "$1" = "-d" ] || [ "$1" = "--debug" ] ;
    then
        debug=--debug ;
    else
        name=$1
        extci_name=ExtCI_at_$1
        ns_name=NS_at_$1
    fi ;

    shift ;
done

# check if name is set
if [ -z "$name" ] ;
then
    if [ -n "$debug" ] ;
    then
        echo "WARNING: Will use default name for CCM ServerActivator."
    fi ;

    hostname=`hostname`
    name=SA_at_$hostname ;
fi

# delegate to corba_runtime_stop script
@ECM.install.bin.dir@/corba_runtime_stop.sh $debug $name

# stop extci
@ECM.install.bin.dir@/extci_stop.sh $debug $extci_name

# local NS
@ECM.install.bin.dir@/ns_stop.sh $debug $ns_name
