#! /bin/sh
# ==================================================================== 
#
#   ECM: The Extensible Container Model
#   Copyright (C) 2004 THALES
#   Contact: openccm-ecm@objectweb.org
#
#   This library is free software; you can redistribute it and/or
#   modify it under the terms of the GNU Lesser General Public
#   License as published by the Free Software Foundation; either
#   version 2.1 of the License, or any later version.
#
#   This library is distributed in the hope that it will be useful,
#   but WITHOUT ANY WARRANTY; without even the implied warranty of
#   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
#   Lesser General Public License for more details.
#
#   You should have received a copy of the GNU Lesser General Public
#   License along with this library; if not, write to the Free Software
#   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
#   USA
#
#   Initial developer(s): Mathieu Vadet
#   Initial funding: IST COACH European Project (IST-2001-34445)
#                    http://www.ist-coach.org
#
# ==================================================================== 


#
#  Starts a CORBA CosNaming service
#

usage() {
    echo "usage: ns_start.sh [options]"
    echo "Starts a CORBA CosNaming service."
    echo "--"
    echo "The following options are supported:"
    echo "  -h, --help     Print this message"
    echo "  -d, --debug    Activate debug mode (optional)"
    echo "  name           The logical name of the CORBA CosNaming service, NS_at_hostname by default (optional)"
    exit 1 ;
}

# parse arguments
name=
debug=

while [ -n "$1" ] ;
do
    if [ "$1" = "-h" ] || [ "$1" = "--help" ] ;
    then
        usage ;
    elif [ "$1" = "-d" ] || [ "$1" = "--debug" ] ;
    then 
        debug=t ;
    else
        name=$1
    fi ;

    shift ;
done

# check if name is set
if [ -z "$name" ] ;
then
    if [ -n "$debug" ] ;
    then
        echo "WARNING: Will use default name for CORBA CosNaming service."
    fi ;

    hostname=`hostname`
    name=NS_at_$hostname ;
fi


# check if the CORBA runtime is started
if [ -f $CORBA_RUNTIME_REPOSITORY/$name.pid ] ;
then
    if [ -n "$debug" ] ;
    then
        echo "WARNING: The CORBA CosNaming service with logical name: $name is already started."
    fi ;
fi

if [ -n "$debug" ] ;
then
    echo "INFO: Starting ORBacus 4.1.0 CORBA CosNaming service ..."
fi

@OpenCCM.install.bin.dir@/launcher.sh fork "" "@ECM.install.launcher.dir@/ns.xml" $CORBA_RUNTIME_REPOSITORY/$name.ior $CORBA_RUNTIME_REPOSITORY/$name.pid

if [ -n "$debug" ] ;
then
    echo "INFO: ORBacus 4.1.0 CORBA CosNaming service started."
fi
