@echo off
REM ==================================================================== 
REM
REM   ECM: The Extensible Container Model
REM   Copyright (C) 2004 THALES
REM   Contact: openccm-ecm@objectweb.org
REM
REM   This library is free software; you can redistribute it and/or
REM   modify it under the terms of the GNU Lesser General Public
REM   License as published by the Free Software Foundation; either
REM   version 2.1 of the License, or any later version.
REM
REM   This library is distributed in the hope that it will be useful,
REM   but WITHOUT ANY WARRANTY; without even the implied warranty of
REM   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
REM   Lesser General Public License for more details.
REM
REM   You should have received a copy of the GNU Lesser General Public
REM   License along with this library; if not, write to the Free Software
REM   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
REM   USA
REM
REM   Initial developer(s): Mathieu Vadet
REM   Initial funding: IST COACH European Project (IST-2001-34445)
REM                    http://www.ist-coach.org
REM
REM ==================================================================== 


REM
REM  Starts a CORBA CosNaming GUI
REM

setlocal

goto begin

:usage
    echo usage: ns_gui [options]
    echo Starts a CORBA CosNaming service.
    echo --
    echo The following options are supported:
    echo   /h, /help    Print this message
    echo   /debug       Activate debug mode (optional)
    echo   name         The logical name of the CORBA CosNaming service to which the GUI is bound, NS_at_hostname by default (optional)
    echo                NOTE: the logical name of the GUI is NSConsole_at_hostname by default or NSConsole_at_name if name is set
    exit /B 1

:begin
set name=
set cname=
set debug=

:parse_args
if not "%1"=="" (
    if "%1"=="/h" (
        goto usage
    ) else if "%1"=="/help" (
        goto usage
    ) else if "%1"=="/debug" (
        set debug=/debug
    ) else (
        set name=%1
        set cname=NSConsole_at_%1
    )

    shift
    goto parse_args
)

REM check if name is set
if not defined name (
    if defined debug (
        echo WARNING: Will use default name for CORBA CosNaming service.
    )

    hostname > %TEMP%\hostname.tmp
    for /f %%i in (%TEMP%\hostname.tmp) do (
        set name=NS_at_%%i
        set cname=NSConsole_at_%%i
    )
    del %TEMP%\hostname.tmp 2> nul
)

REM check if the service is started
if exist %CORBA_RUNTIME_REPOSITORY%\%name%.pid (
    if defined debug (
        echo WARNING: The CORBA CosNaming service with logical name: %name% is already started.
    )
)

if defined debug (
    echo INFO: starting ORBacus 4.1.0 CORBA CosNaming GUI ...
)

REM obtain CosNaming IOR
@ECM.install.bin.dir@\ns_ior.bat %debug% /url %name%  > %TEMP%\nsior.tmp
for /f %%i in (%TEMP%\nsior.tmp) do (
    set nsior=%%i
)
del %TEMP%\nsior.tmp 2> nul

set jvmargs=-Dior_ns=%nsior%
set args=--runid gui

call @OpenCCM.install.bin.dir@\launcher.bat exec "%jvmargs%" "%args% @ECM.install.launcher.dir@/ns.xml" %CORBA_RUNTIME_REPOSITORY%\%name%Console.ior %CORBA_RUNTIME_REPOSITORY%\%name%Console.pid

if defined debug (
    echo INFO: ORBacus 4.1.0 CORBA CosNaming GUI started.
)

:eof
