// ====================================================================
// 
//   ECM: The Extensible Container Model
//   Copyright (C) 2004 THALES
//   Contact: openccm-ecm@objectweb.org
//
//   This library is free software; you can redistribute it and/or
//   modify it under the terms of the GNU Lesser General Public
//   License as published by the Free Software Foundation; either
//   version 2.1 of the License, or any later version.
//
//   This library is distributed in the hope that it will be useful,
//   but WITHOUT ANY WARRANTY; without even the implied warranty of
//   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
//   Lesser General Public License for more details.
//
//   You should have received a copy of the GNU Lesser General Public
//   License along with this library; if not, write to the Free Software
//   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
//   USA
//
//   Initial developer(s): Mathieu Vadet.
//   Initial Funding: IST COACH European project (IST-2001-34445)
//                    http://www.ist-coach.org 
//
// ====================================================================



package org.objectweb.ccm.runtime;

import org.objectweb.corba.runtime.*;

/**
 **  <p>Base class for server-side interceptors.</p>
 **
 **  <p><i>NOTE: </i>not used, the generated interceptors directly includes this code, so that the
 **  inheritance link is available and can be used to inherit from the generated CORBA skeleton.</p>
 **/
abstract public class CallInterceptorSkBase
{
    // call interceptor skeleton
    static private String _class_name = "CallInterceptorSkBase";
    private ORBService _orb_service;
    private ServicesSet _services_set;
    private String _uuid;

    private org.omg.CORBA.Object _delegate;
    private ServerCallContextImpl _call_ctx;
    private java.util.Hashtable _informations;

    // default constructor
    protected
    CallInterceptorSkBase(ORBService orbs, ServicesSet sset, String uuid,
                          org.omg.CORBA.Object executor)
    {
        // call interceptor sk
        _orb_service = orbs;
        _services_set = sset;
        _uuid = uuid;
        _delegate = executor;
        _informations = new java.util.Hashtable();

        // create call context
        _call_ctx = new ServerCallContextImpl(_orb_service, _services_set, executor);
    }

    //
    // internal  operations
    //

    final protected ORBService
    _getORBService()
    {
        return _orb_service;
    }

    final protected ServerCallContextImpl
    _getCallCtx()
    {
        return _call_ctx;
    }

    final protected CallInformation
    _getCallInformation(String sname)
    {
        // check if information is already computed
        CallInformation info = (CallInformation)_informations.get(sname);

        if (info==null) {
            // create information
            info = new CallInformation();
            // store
            _informations.put(sname, info);
        }

        // obtain policies in effect
        // check if policies are already computed
        if (info.policies==null) {
            info.policies = _services_set.get_policies(_uuid, sname);
        }

        // NOTE: controllers are created on demand in the CallInformation class

        return info;
    }
 
    final protected short
    _preinvoke(org.coach.ECA.ServerCallController[] ctrls)
    {
        final String opname = "_preinvoke";
        org.omg.CORBA.Any curr_exc = null;
        org.omg.CORBA.Any curr_res = null;

        // initialization
        short fstate = FlowState.RECEIVE_REQUEST;
        int ctrl_idx = 0;
        boolean finished = false;
        org.coach.ECA.ServerCallController ctrl = null;

        while ((!finished) && (ctrl_idx<ctrls.length)){
            // check if a system exception was set
            curr_exc = _getCallCtx().system_exception();
            if (curr_exc!=null) {
                // change flow state
                fstate = FlowState.SEND_SYSTEM_EXCEPTION;
            }

            // check if a user exception was set
            curr_exc = _getCallCtx().user_exception();
            if (curr_exc!=null) {
                // change flow state
                fstate = FlowState.SEND_USER_EXCEPTION;
            }

            // check if a result was set
            curr_res = _getCallCtx().result();
            if (curr_res!=null) {
                // not possible in preinvoke as result is not modifiable by controllers
                final String msg = "FAILED (unexpected state in preinvoke: SEND_REPLY)";
                TheLogger.error(_class_name, opname, msg);
            }

            // we are on the receive_request IP
            if (fstate==FlowState.RECEIVE_REQUEST) {
                // take next controller
                ctrl = ctrls[ctrl_idx++];
                ctrl.receive_request(_getCallCtx());

                // push completed controller on the flow stack (starting IP)
                _getCallCtx().pushController(ctrl);
            }
            // check if there's anymore controllers on the flow stack
            else if (_getCallCtx().isEmpty()) {
                finished = true;
            }
            // we are on the send_system_exception IP
            else if (fstate==FlowState.SEND_SYSTEM_EXCEPTION) {
                // get controller from flow stack (ending IP)
                ctrl = _getCallCtx().popController();
                ctrl.send_system_exception(_getCallCtx());
            }
            // we are on the send_user_exception IP
            else if (fstate==FlowState.SEND_SYSTEM_EXCEPTION) {
                // get controller from flow stack (ending IP)
                ctrl = _getCallCtx().popController();
                ctrl.send_user_exception(_getCallCtx());
            }
            // we are on the send_reply IP
            else {
                // not possible in preinvoke as result is not modifiable by controllers
                final String msg = "FAILED (unexpected state in preinvoke: SEND_REPLY)";
                TheLogger.error(_class_name, opname, msg);
            }
        }

        // return flow state
        return fstate;
    }

    final protected short
    _postinvoke(short fstate)
    {
        org.omg.CORBA.Any curr_exc = null;
        org.omg.CORBA.Any curr_res = null;
        boolean finished = false;
        org.coach.ECA.ServerCallController ctrl = null;

        // intialization of the flow state to take into account
        // business code actions

        // check if a system exception was set
        curr_exc = _getCallCtx().system_exception();
        curr_exc = _getCallCtx().user_exception();
        if (curr_exc!=null) {
            // change flow state
            fstate = FlowState.SEND_SYSTEM_EXCEPTION;
        }
        // check if a user exception was set
        else if (curr_exc!=null) {
            // change flow state
            fstate = FlowState.SEND_USER_EXCEPTION;
        }
        else {
            // change flow state
            fstate = FlowState.SEND_REPLY;
        }

        while (!finished) {
            // check if there's anymore controllers on the flow stack
            if (_getCallCtx().isEmpty()) {
                finished = true;
            }
            // we are on the send_system_exception IP
            else if (fstate==FlowState.SEND_SYSTEM_EXCEPTION) {
                // get controller from flow stack (ending IP)
                ctrl = _getCallCtx().popController();
                ctrl.send_system_exception(_getCallCtx());
            }
            // we are on the send_user_exception IP
            else if (fstate==FlowState.SEND_SYSTEM_EXCEPTION) {
                // get controller from flow stack (ending IP)
                ctrl = _getCallCtx().popController();
                ctrl.send_user_exception(_getCallCtx());
            }
            // we are on the send_reply IP
            else {
                // get controller from flow stack (ending IP)
                ctrl = _getCallCtx().popController();
                ctrl.send_reply(_getCallCtx());
            }

            // check if a system exception was set
            curr_exc = _getCallCtx().system_exception();
            if (curr_exc!=null) {
                // change flow state
                fstate = FlowState.SEND_SYSTEM_EXCEPTION;
            }

            // check if a user exception was set
            curr_exc = _getCallCtx().user_exception();
            if (curr_exc!=null) {
                // change flow state
                fstate = FlowState.SEND_USER_EXCEPTION;
            }

            // check if a result was set
            curr_res = _getCallCtx().result();
            if (curr_res!=null) {
                // change flow state
                fstate = FlowState.SEND_REPLY;
            }
        }

        // return flow state
        return fstate;
    }
}