// ====================================================================
// 
//   ECM: The Extensible Container Model
//   Copyright (C) 2004 THALES
//   Contact: openccm-ecm@objectweb.org
//
//   This library is free software; you can redistribute it and/or
//   modify it under the terms of the GNU Lesser General Public
//   License as published by the Free Software Foundation; either
//   version 2.1 of the License, or any later version.
//
//   This library is distributed in the hope that it will be useful,
//   but WITHOUT ANY WARRANTY; without even the implied warranty of
//   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
//   Lesser General Public License for more details.
//
//   You should have received a copy of the GNU Lesser General Public
//   License along with this library; if not, write to the Free Software
//   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
//   USA
//
//   Initial developer(s): Mathieu Vadet.
//   Initial Funding: IST COACH European project (IST-2001-34445)
//                    http://www.ist-coach.org 
//
// ====================================================================



package org.objectweb.ccm.runtime;

import org.objectweb.corba.runtime.*;

/**
 **  <p>.</p>
 **/
public class EventConsumerBaseStubI
extends org.omg.Components._EventConsumerBaseStub
{
    // consumer base stub
    static private String _class_name = "EventConsumerBaseStubI";
    private ORBService _orb_service;
    private ServicesSet _services_set;
    private String _uuid;
    private ClientCallContextImpl _call_ctx;
    private java.util.Hashtable _informations;
    private org.omg.Components.EventConsumerBase _delegate;
    private String _scoped_name;

    // default constructor
    public
    EventConsumerBaseStubI(ORBService orbs, ServicesSet sset,
                           String uuid, org.omg.CORBA.Object del,
                           String sname)
    {
        // consumer base stub
        _orb_service = orbs;
        _services_set = sset;
        _uuid = uuid;
        _delegate = org.omg.Components.EventConsumerBaseHelper.narrow(del);
        _informations = new java.util.Hashtable();
        _scoped_name = sname;

        // create call context
        _call_ctx = new ClientCallContextImpl(_orb_service, _services_set, del);
    }

    //
    //  internal operations
    //

    private CallInformation
    _getCallInformation(String name) 
    {
        // check if information is already computed
        CallInformation info = (CallInformation)_informations.get(name);
        
        if (info==null) {
            // create information
            info = new CallInformation();
            // store
            _informations.put(name, info);
        }
        
        // obtain policies in effect
        // check if policies are already computed
        if (info.policies==null) {
            info.policies = _services_set.get_policies(_uuid, name);
        }
        
        // NOTE: controllers are created on demand in the CallInformation class
        
        return info;
    }
    
    private short
    _preinvoke(org.coach.ECA.ClientCallController[] ctrls) 
    {
        final String opname = "_preinvoke";
        org.omg.CORBA.Any curr_exc = null;
        org.omg.CORBA.Any curr_res = null;
        
        // initialization
        short fstate = FlowState.SEND_REQUEST;
        int ctrl_idx = 0;
        boolean finished = false;
        org.coach.ECA.ClientCallController ctrl = null;
        
        while ((!finished) && (ctrl_idx<ctrls.length)){
            // check if a system exception was set
            curr_exc = _call_ctx.system_exception();
            if (curr_exc!=null) {
                // change flow state
                fstate = FlowState.RECEIVE_SYSTEM_EXCEPTION;
            }
        
            // check if a user exception was set
            curr_exc = _call_ctx.user_exception();
            if (curr_exc!=null) {
                // change flow state
                fstate = FlowState.RECEIVE_USER_EXCEPTION;
            }
        
            // check if a result was set
            curr_res = _call_ctx.result();
            if (curr_res!=null) {
                // not possible in preinvoke as result is not modifiable by controllers
                final String msg = "FAILED (unexpected state in preinvoke: RECEIVE_REPLY)";
                TheLogger.error(_class_name, opname, msg);
            }
        
            // we are on the send_request IP
            if (fstate==FlowState.SEND_REQUEST) {
                // take next controller
                ctrl = ctrls[ctrl_idx++];
                ctrl.send_request(_call_ctx);
        
                // push completed controller on the flow stack (starting IP)
                _call_ctx.pushController(ctrl);
            }
            // check if there's anymore controllers on the flow stack
            else if (_call_ctx.isEmpty()) {
                finished = true;
            }
            // we are on the receive_system_exception IP
            else if (fstate==FlowState.RECEIVE_SYSTEM_EXCEPTION) {
                // get controller from flow stack (ending IP)
                ctrl = _call_ctx.popController();
                ctrl.receive_system_exception(_call_ctx);
            }
            // we are on the receive_user_exception IP
            else if (fstate==FlowState.RECEIVE_SYSTEM_EXCEPTION) {
                // get controller from flow stack (ending IP)
                ctrl = _call_ctx.popController();
                ctrl.receive_user_exception(_call_ctx);
            }
            // we are on the receive_reply IP
            else {
                // not possible in preinvoke as result is not modifiable by controllers
                final String msg = "FAILED (unexpected state in preinvoke: RECEIVE_REPLY)";
                TheLogger.error(_class_name, opname, msg);
            }
        }
        
        // return flow state
        return fstate;
    }
    
    /**
     * .
     */
    private short
    _postinvoke(short fstate) 
    {
        org.omg.CORBA.Any curr_exc = null;
        org.omg.CORBA.Any curr_res = null;
        boolean finished = false;
        org.coach.ECA.ClientCallController ctrl = null;
        
        // intialization of the flow state to take into account
        // business code actions
        
        // check if a system exception was set
        curr_exc = _call_ctx.system_exception();
        curr_exc = _call_ctx.user_exception();
        if (curr_exc!=null) {
            // change flow state
            fstate = FlowState.RECEIVE_SYSTEM_EXCEPTION;
        }
        // check if a user exception was set
        else if (curr_exc!=null) {
            // change flow state
            fstate = FlowState.RECEIVE_USER_EXCEPTION;
        }
        else {
            // change flow state
            fstate = FlowState.RECEIVE_REPLY;
        }
        
        while (!finished) {
            // check if there's anymore controllers on the flow stack
            if (_call_ctx.isEmpty()) {
                finished = true;
            }
            // we are on the receive_system_exception IP
            else if (fstate==FlowState.RECEIVE_SYSTEM_EXCEPTION) {
                // get controller from flow stack (ending IP)
                ctrl = _call_ctx.popController();
                ctrl.receive_system_exception(_call_ctx);
            }
            // we are on the receive_user_exception IP
            else if (fstate==FlowState.RECEIVE_SYSTEM_EXCEPTION) {
                // get controller from flow stack (ending IP)
                ctrl = _call_ctx.popController();
                ctrl.receive_user_exception(_call_ctx);
            }
            // we are on the receive_reply IP
            else {
                // get controller from flow stack (ending IP)
                ctrl = _call_ctx.popController();
                ctrl.receive_reply(_call_ctx);
            }
        
            // check if a system exception was set
            curr_exc = _call_ctx.system_exception();
            if (curr_exc!=null) {
                // change flow state
                fstate = FlowState.RECEIVE_SYSTEM_EXCEPTION;
            }
        
            // check if a user exception was set
            curr_exc = _call_ctx.user_exception();
            if (curr_exc!=null) {
                // change flow state
                fstate = FlowState.RECEIVE_USER_EXCEPTION;
            }
        
            // check if a result was set
            curr_res = _call_ctx.result();
            if (curr_res!=null) {
                // change flow state
                fstate = FlowState.RECEIVE_REPLY;
            }
        }
        
        // return flow state
        return fstate;
    }

    //
    //  IDL:omg.org/Components/EventConsumerBase:1.0
    //

    final public void
    push_event(org.omg.Components.EventBase evt) 
    throws org.omg.Components.BadEventType
    {
        final String opname = "push_event";
        org.omg.CORBA.Any anyexc = null;
        
        // check if controllers are created
        CallInformation callinfo = _getCallInformation(_scoped_name);
        
        // create a new entry in the ClientCallContext
        _call_ctx.addEntry(callinfo, opname);
        
        // call preinvoke routine
        short fstate = _preinvoke(callinfo.getClientCallControllers());
        
        // if flow state is not SEND_REQUEST, then skip business call
        if (fstate==FlowState.RECEIVE_SYSTEM_EXCEPTION) {
            anyexc = _call_ctx.system_exception();
            _call_ctx.removeEntry();
            throw org.omg.CORBA.SystemExceptionHelper.extract(anyexc);
        }
        else if (fstate==FlowState.RECEIVE_USER_EXCEPTION) {
            anyexc = _call_ctx.user_exception();
            _call_ctx.removeEntry();
            // check which exception was raised
            if (anyexc.type()==org.omg.Components.BadEventTypeHelper.type()) {
                throw org.omg.Components.BadEventTypeHelper.extract(anyexc);
            }
            else {
                TheLogger.error(_class_name, opname, "unknown user exception was raised");
            }
        }
        else  if (fstate==FlowState.RECEIVE_REPLY) {
            // not possible in preinvoke as result is not modifiable by controllers
            TheLogger.error(_class_name, opname, "unexpected state after preinvoke: RECEIVE_REPLY");
        }
        
        try {
            // call business op
            _delegate.push_event(evt);
        }
        catch (org.omg.CORBA.SystemException ex) {
            anyexc = _call_ctx.create_any();
            org.omg.CORBA.SystemExceptionHelper.insert(anyexc, ex);
            _call_ctx.send_system_exception(anyexc);
        }
        catch (org.omg.Components.BadEventType ex) {
            // set info in context
            anyexc = _orb_service.create_any();
            org.omg.Components.BadEventTypeHelper.insert(anyexc, ex);
            _call_ctx.send_user_exception(anyexc);
        }
        // call postinvoke routine
        fstate = _postinvoke(fstate);
        
        // check final flow state
        if (fstate==FlowState.RECEIVE_SYSTEM_EXCEPTION) {
            anyexc = _call_ctx.system_exception();
            _call_ctx.removeEntry();
            throw org.omg.CORBA.SystemExceptionHelper.extract(anyexc);
        }
        else if (fstate==FlowState.RECEIVE_USER_EXCEPTION) {
            anyexc = _call_ctx.user_exception();
            _call_ctx.removeEntry();
            // check which exception was raised
            if (anyexc.type()==org.omg.Components.BadEventTypeHelper.type()) {
                throw org.omg.Components.BadEventTypeHelper.extract(anyexc);
            }
            else {
                TheLogger.error(_class_name, opname, "unknown user exception was raised");
            }
        }
        else if (fstate==FlowState.RECEIVE_REPLY) {
            _call_ctx.removeEntry();
        }
    }
}
