// ====================================================================
// 
//   ECM: The Extensible Container Model
//   Copyright (C) 2004 THALES
//   Contact: openccm-ecm@objectweb.org
//
//   This library is free software; you can redistribute it and/or
//   modify it under the terms of the GNU Lesser General Public
//   License as published by the Free Software Foundation; either
//   version 2.1 of the License, or any later version.
//
//   This library is distributed in the hope that it will be useful,
//   but WITHOUT ANY WARRANTY; without even the implied warranty of
//   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
//   Lesser General Public License for more details.
//
//   You should have received a copy of the GNU Lesser General Public
//   License along with this library; if not, write to the Free Software
//   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
//   USA
//
//   Initial developer(s): Mathieu Vadet.
//   Initial Funding: IST COACH European project (IST-2001-34445)
//                    http://www.ist-coach.org 
//
// ====================================================================



package org.objectweb.ecm.taskdefs;

/**
 **  <p>Generates the container interceptors for a OMG IDL3 declaration.</p>
 **
 **  <p><bold>Parameters</bold></p>
 **  <tr>
 **    <td><bold>Attributes</bold></td>
 **    <td><bold>Description</bold></td>
 **    <td><bold>Required</bold></td>
 **  </tr>
 **  <tr>
 **    <td>scope</td>
 **    <td>The scoped name of the OMG IDL3 declaration for which the interceptors are
 **        generated. The declaration must be a module or a component.</td>
 **    <td>Yes</td>
 **  </tr>
 **  <tr>
 **    <td>generateto</td>
 **    <td>Directory of the generated interceptors.</td>
 **    <td>Yes</td>
 **  </tr>
 **  <tr>
 **    <td>cfgdir</td>
 **    <td>The directory corresponding to the OpenCCM_CONFIG_DIR environment variable and where the
 **        IR3-related executables of OpenCCM store and lookup  IORs, PIDs, etc. Can be set to a 
 **        temporary directory.</td>
 **    <td>Yes</td>
 **  </tr>
 **
 **  <tr>
 **    <td><bold>Properties</bold></td>
 **    <td><bold>Description</bold></td>
 **    <td><bold>Required</bold></td>
 **  </tr>
 **  <tr>
 **    <td>ECM.install.dir</td>
 **    <td>Location of the ECM installation directory.</td>
 **    <td>Yes</td>
 **  </tr>
 **
 **  <p>NOTE: the OpenCCM IR3 must be started before calling this task.</p>
 **
 **  @see org.objectweb.corba.build.IR3StartTask IR3StartTask
 **/
public class CIDLCOPITask
extends org.apache.tools.ant.Task
{
    // params
    private String _scope;
    private java.io.File _generateto;
    private java.io.File _cfgdir;

    // default constructor
    public
    CIDLCOPITask()
    {
        // params
        _scope = null;
        _generateto = null;
        _cfgdir = null;
    }

    //
    //  internal operations
    //


    // NOTE: check required attributes, properties, environment, ...
    private void
    validate()
    throws org.apache.tools.ant.BuildException
    {
        String msg = "";

        // check OS family
        if (!OSHelper.isUnix() && !OSHelper.isWindows()) {
            msg = "target os must be of unix or windows family";
            throw new org.apache.tools.ant.BuildException(msg);
        }

        // check scope
        if (_scope==null) {
            msg = "scope attribute is missing";
            throw new org.apache.tools.ant.BuildException(msg);
        }

        // check generateto
        if (_generateto==null) {
            msg = "generateto attribute is missing";
            throw new org.apache.tools.ant.BuildException(msg);
        }
        if ((!_generateto.exists()) || (!_generateto.isDirectory())) {
            msg = "Generation directory does not exist or is not a directory";
            throw new org.apache.tools.ant.BuildException(msg);
        }

        // check cfgdir
        if (_cfgdir==null) {
            msg = "cfgdir attribute is missing";
            throw new org.apache.tools.ant.BuildException(msg);
        }
        if ((!_cfgdir.exists()) || (!_cfgdir.isDirectory())) {
            msg = "Configuration directory does not exist or is not a directory";
            throw new org.apache.tools.ant.BuildException(msg);
        }

        // check "ECM.install.dir" property
        String instdir = getProject().getProperty("ECM.install.dir");
        if (instdir==null) {
            msg = "ECM.install.dir property must be set";
            throw new org.apache.tools.ant.BuildException(msg);
        }
    }

    private void
    cidltocopi(java.io.File cfgdir,
               String scope,
               java.io.File gento)
    throws org.apache.tools.ant.BuildException
    {
        // uses exec ant task to execute the command
        org.apache.tools.ant.taskdefs.ExecTask cmd = null;
        cmd = (org.apache.tools.ant.taskdefs.ExecTask)getProject().createTask("exec");

        // convert ECM.install.dir path
        OSHelper.pathConvert(getProject(), "os.ECM.install.dir", "ECM.install.dir");
        // build executable name
        String cmdname = null;
        String instdir = getProject().getProperty("os.ECM.install.dir");
        if (OSHelper.isUnix()) {
            cmdname = instdir+"/bin/cidl_copi.sh";
        }
        else if (OSHelper.isWindows()) {
            cmdname = instdir+"\\bin\\cidl_copi.bat";
        }

        cmd.setExecutable(cmdname);

        // add environment variables
        org.apache.tools.ant.types.Environment.Variable openccm_homedir = null;
        org.apache.tools.ant.types.Environment.Variable openccm_cfgdir = null;

        // convert OpenCCM.install.dir path
        OSHelper.pathConvert(getProject(), "os.OpenCCM.install.dir", "OpenCCM.install.dir");
        String homedir = getProject().getProperty("os.OpenCCM.install.dir");
        openccm_homedir = new org.apache.tools.ant.types.Environment.Variable();
        openccm_homedir.setKey("OpenCCM_HOMEDIR");
        openccm_homedir.setValue(homedir);
        cmd.addEnv(openccm_homedir);

        openccm_cfgdir = new org.apache.tools.ant.types.Environment.Variable();
        openccm_cfgdir.setKey("OpenCCM_CONFIG_DIR");
        openccm_cfgdir.setValue(cfgdir.getPath());
        cmd.addEnv(openccm_cfgdir);

        // add generation directory as an argument
        org.apache.tools.ant.types.Commandline.Argument arg = cmd.createArg();
        arg.setLine(" -d "+gento);

        arg = cmd.createArg();
        arg.setValue(scope);

        // launch
        cmd.execute();
    }

    //
    //  attribute setters
    //

    final public void
    setGenerateto(java.io.File dir)
    {
        _generateto = dir;
    }

    final public void
    setScope(String scope)
    {
        _scope = scope;
    }

    final public void
    setCfgdir(java.io.File cfgdir)
    {
        _cfgdir = cfgdir;
    }

    //
    //  org.apache.tools.ant.Task
    //

    final public void
    execute()
    throws org.apache.tools.ant.BuildException
    {
        validate();

        // NOTE: should check if generation is needed
        cidltocopi(_cfgdir, _scope, _generateto);
    }
}
