// ====================================================================
// 
//   ECM: The Extensible Container Model
//   Copyright (C) 2004 THALES
//   Contact: openccm-ecm@objectweb.org
//
//   This library is free software; you can redistribute it and/or
//   modify it under the terms of the GNU Lesser General Public
//   License as published by the Free Software Foundation; either
//   version 2.1 of the License, or any later version.
//
//   This library is distributed in the hope that it will be useful,
//   but WITHOUT ANY WARRANTY; without even the implied warranty of
//   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
//   Lesser General Public License for more details.
//
//   You should have received a copy of the GNU Lesser General Public
//   License along with this library; if not, write to the Free Software
//   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
//   USA
//
//   Initial developer(s): Mathieu Vadet.
//   Initial Funding: IST COACH European project (IST-2001-34445)
//                    http://www.ist-coach.org 
//
// ====================================================================



package org.objectweb.ecm.taskdefs;

/**
 **  <p>Build a jar archive from a source directory.</p>
 **
 **  <p><bold>Parameters</bold></p>
 **  <tr>
 **    <td><bold>Attribute</bold></td>
 **    <td><bold>Description</bold></td>
 **    <td><bold>Required</bold></td>
 **  </tr>
 **  <tr>
 **    <td>idlfile</td>
 **    <td>The OMG IDL file to be compiled.</td>
 **    <td>Yes</td>
 **  </tr>
 **  <tr>
 **    <td>generateto</td>
 **    <td>Directory of the generated files.</td>
 **    <td>Yes</td>
 **  </tr>
 **  <tr>
 **    <td>genfile</td>
 **    <td>A generated file corresponding to a IDL declaration of the 
 **        compiled OMG IDL file. This file is used to perform an uptodate check.
 **        <i>NOTE: genfile must be relative to generateto</i></td>
 **    <td>No, if not set, then the corresponding uptodate check is ignored.</td>
 **  </tr>
 **  <tr>
 **    <td>idldeps</td>
 **    <td>A list of OMG IDL files which are dependencies of the compiled OMG IDL file.
 **        This list is used to peform uptodate checks.<br>
 **        <i>NOTE: not implemented, use nested element instead.</i></td>
 **    <td>No, if not set, then the corresponding uptodate check is ignored.</td>
 **  </tr>
 **  <tr>
 **    <td>includedirs</td>
 **    <td>A list of directories used to lookup for included OMG IDL files.<br>
 **        <i>NOTE: not implemented, use nested element instead.</i></td>
 **    <td>No.</td>
 **  </tr>
 **  <tr>
 **    <td>includedir</td>
 **    <td>A directory name used to lookup for included OMG IDL files.<br>
 **        <i>NOTE: not implemented, use nested element instead.</i></td>
 **    <td>No.</td>
 **  </tr>
 **  <tr>
 **    <td>prefix</td>
 **    <td>A prefix which will be appended to the Java package name.</td>
 **    <td>No.</td>
 **  </tr>
 **  <tr>
 **    <td>archivename</td>
 **    <td>Name of the built archive.</td>
 **    <td>No, if not set, then no archive is built.</td>
 **  </tr>
 **  <tr>
 **    <td>compileto</td>
 **    <td>Directory of the class files.</td>
 **    <td>No, unless the <tt>archivename</tt> attribute is set.</td>
 **  </tr>
 **  <tr>
 **    <td>buildto</td>
 **    <td>Directory of the archive (jar) file.</td>
 **    <td>No, unless the <tt>archivename</tt> attribute is set.</td>
 **  </tr>
 **  <tr>
 **    <td>classpathref (or classpathRef)</td>
 **    <td>The reference to the classpath used for compilation.</td>
 **    <td>No</td>
 **  </tr>
 **  <tr>
 **    <td>bootclasspathref (or bootclasspathRef)</td>
 **    <td>The reference to the bootclasspath used for compilation.</td>
 **    <td>Yes, unless a nested <tt>bootclasspath</tt> element is present.</td>
 **  </tr>
 **
 **  <p><bold>Parameters specified as nested elements</bold></p>
 **  <p>This task support the <tt>classpath</tt> and <tt>bootclasspath</tt> attributes also
 **  as nested elements in the form of path-like structures (unlike the attributes which are
 **  path references).</p>
 ** 
 **  <p>This task support the <tt>idldeps</tt> and <tt>includedirs</tt> attributes also as nested
 **  elements in the form of <tt>fileset</tt> element.</p>
 ** 
 **  <p>This task support the <tt>idldep</tt> and <tt>includedir</tt> (without an 's') nested elements
 **  in the form of a file name.</p>
 **/
abstract public class IDLtoJavaTask
extends org.apache.tools.ant.Task
{
    //
    private java.io.File _idlfile;
    private java.io.File _generateto;
    private java.io.File _genfile;
    private String _genfilename;
    private java.util.ArrayList _idldeps;
    private java.util.ArrayList _idldep;
    private java.util.ArrayList _includedirs;
    private java.util.ArrayList _includedir;
    private String _prefix;

    private String _archive;    
    private java.io.File _buildto;
    private java.io.File _compileto;
    private BuildJarTask _buildjar;

    // default constructor
    protected
    IDLtoJavaTask()
    {
        //
        _idlfile = null;
        _generateto = null;
        _genfile = null;
        _genfilename = null;
        _idldeps = new java.util.ArrayList();
        _idldep = new java.util.ArrayList();
        _includedirs = new java.util.ArrayList();
        _includedir = new java.util.ArrayList();
        _prefix = null;

        _archive = null;
        _buildto = null;
        _compileto = null;
        _buildjar = null;
    }

    //
    //  internal operations
    //

    private BuildJarTask
    buildjar()
    {
        if (_buildjar==null) {
            org.apache.tools.ant.Task task = getProject().createTask("buildjar");
            _buildjar = (BuildJarTask)task;
        }

        return _buildjar;
    }

    // NOTE: check required attributes
    private void
    validateAttributes()
    throws org.apache.tools.ant.BuildException
    {
        String msg = "";
        if (_idlfile==null) {
            msg = "idlfile attribute is missing";
            throw new org.apache.tools.ant.BuildException(msg);
        }

        if ((!_idlfile.exists()) || (!_idlfile.isFile())) {
            msg = "Source OMG IDL file does not exists or is not a file";
            throw new org.apache.tools.ant.BuildException(msg);
        }

        if ((!_generateto.exists()) || (!_generateto.isDirectory())) {
            msg = "Generation directory does not exist or is not a directory";
            throw new org.apache.tools.ant.BuildException(msg);
        }

        // compute _genfile
        _genfile = new java.io.File(_generateto, _genfilename.replace('/', java.io.File.separatorChar));
        if (_genfile==null) {
            msg = "[validateAttributes] genfile (null): "+_generateto.getPath()+"/"+_genfilename.replace('/', java.io.File.separatorChar);
            log(msg);
        }

        // 
        if (_archive!=null) {
            if (_compileto==null) {
                msg = "compileto attribute must be set when archivename attribute is set";
                throw new org.apache.tools.ant.BuildException(msg);
            }

            if ((!_compileto.exists()) || (!_compileto.isDirectory())) {
                msg = "class directory does not exist or is not a directory";
                throw new org.apache.tools.ant.BuildException(msg);
            }

            if (_buildto==null) {
                msg = "buildto attribute must be set when archivename attribute is set";
                throw new org.apache.tools.ant.BuildException(msg);
            }

            if ((!_buildto.exists()) || (!_buildto.isDirectory())) {
                msg = "archive directory does not exist or is not a directory";
                throw new org.apache.tools.ant.BuildException(msg);
            }
        }
    }

    private boolean
    idlfileUpToDate()
    {
        boolean genfile_passed=false;
        boolean idldeps_passed=false;
        boolean idldep_passed=false;

        // check against generated file if any
        if (_genfile!=null) {
            if ((!_genfile.exists()) || (_genfile.lastModified()<_idlfile.lastModified())) {
                return false;
            }
            else {
                genfile_passed=true;
            }
        }

        // check against the IDL dependencies ("idldeps" attribute/nested element)
        org.apache.tools.ant.types.FileSet[] sets = null;
        sets = (org.apache.tools.ant.types.FileSet[])_idldeps.toArray(new org.apache.tools.ant.types.FileSet[0]);

        if (sets.length>0) {
            for (int i=0;i<sets.length;i++) {
                org.apache.tools.ant.DirectoryScanner scanner = sets[i].getDirectoryScanner(super.project);
                String[] filenames = scanner.getIncludedFiles();
                java.io.File file = null;

                log("[idlfileUpToDate] base dir: "+scanner.getBasedir().getPath());
                for (int j=0;j<filenames.length;j++) {
                    log("[idlfileUpToDate] filename: "+filenames[j]);
                    file = new java.io.File(scanner.getBasedir(), filenames[j]);
                    if (_idlfile.lastModified()>file.lastModified()) {
                        return false;
                    }
                }
            }

            idldeps_passed=true;
        }

        // check against the IDL dependencies ("idldep" nested element)
        NestedElementWithFile[] idlfiles = (NestedElementWithFile[])_idldep.toArray(new NestedElementWithFile[0]);
        if (idlfiles.length!=0) {
            for (int i=0;i<idlfiles.length;i++) {
                if (_idlfile.lastModified()>idlfiles[i].getFile().lastModified()) {
                    return false;
                }
            }

            idldep_passed = true;
        }

        // NOTE: if no check is passed, this means that no uptodate check was performed and therefore
        // we are forced to say that the file is not uptodate
        if (genfile_passed || idldeps_passed || idldep_passed) {
            return true;
        }

        return false;
    }

    private boolean
    jarUpToDate(java.io.File tmpdir)
    {
        // check if an archive is requested
        if (_archive==null) {
            return true;
        }

        // check if some source files were compiled
        if (tmpdir.listFiles().length==0) {
            return true;
        }

        return false;
    }

    //
    //  attribute setters
    //

    final public void
    setIdlfile(java.io.File file)
    {
        _idlfile = file;
    }

    final public void
    setGenerateto(java.io.File dir)
    {
        _generateto = dir;
    }

    final public void
    setGenfile(String filename)
    {
        _genfilename = filename;
    }

    final public void
    setIdldeps(String idldeps)
    {
        // TODO
        // NOT IMPLEMENTED
    }

    final public void
    addIdldeps(org.apache.tools.ant.types.FileSet set)
    {
        _idldeps.add(set);
    }

    final public NestedElementWithFile
    createIdldep()
    {
        NestedElementWithFile idldep = new NestedElementWithFile();
        _idldep.add(idldep);
        return idldep;
    }

    final public void
    addIncludedirs(org.apache.tools.ant.types.DirSet set)
    {
        _includedirs.add(set);
    }

    final public NestedElementWithFile
    createIncludedir()
    {
        NestedElementWithFile dir = new NestedElementWithFile();
        _includedir.add(dir);
        return dir;
    }

    final public void
    setPrefix(String p)
    {
        _prefix = p;
    }

    final public void
    setCompileto(java.io.File dir)
    {
        _compileto = dir;
    }

    final public void
    setArchivename(String name)
    {
        _archive = name;
    }

    final public void
    setBuildto(java.io.File dir)
    {
        _buildto = dir;
    }

    final public void
    setClasspathRef(org.apache.tools.ant.types.Reference cp)
    {
        buildjar().setClasspathRef(cp);
    }

    final public void
    setClasspathref(org.apache.tools.ant.types.Reference cp)
    {
        buildjar().setClasspathref(cp);
    }

    final public void
    setBootclasspathRef(org.apache.tools.ant.types.Reference cp)
    {
        buildjar().setBootclasspathRef(cp);
    }

    final public void
    setBootclasspathref(org.apache.tools.ant.types.Reference cp)
    {
        buildjar().setBootclasspathref(cp);
    }

    final public org.apache.tools.ant.types.Path
    createClasspath()
    {
        return buildjar().createClasspath();
    }

    final public org.apache.tools.ant.types.Path
    createBootclasspath()
    {
        return buildjar().createBootclasspath();
    }

    //
    //  org.apache.tools.ant.Task
    //

    final public void
    execute()
    throws org.apache.tools.ant.BuildException
    {
        // 
        validateAttributes();

        // create temp directory
        java.io.File tmpdir = FileHelper.createTempDir();

        // compile IDL file
        if (!idlfileUpToDate()) {
            log("[execute] OMG IDL file is not uptodate: "+_idlfile.getName());

            java.util.ArrayList vdirs = new java.util.ArrayList();

            // add dir sets
            org.apache.tools.ant.types.DirSet[] sets = null;
            sets = (org.apache.tools.ant.types.DirSet[])_includedirs.toArray(new org.apache.tools.ant.types.DirSet[0]);

            for (int i=0;i<sets.length;i++) {
                org.apache.tools.ant.DirectoryScanner scanner = sets[i].getDirectoryScanner(super.project);
                String[] dirnames = scanner.getIncludedDirectories();

                for (int j=0;j<dirnames.length;j++) {
                    java.io.File dir = new java.io.File(scanner.getBasedir(), dirnames[j]);
                    if (!dir.exists()) {
                        String msg = "Included directory does not exists: "+dir.getPath();
                        log(msg, org.apache.tools.ant.Project.MSG_WARN);
                    }
                    else if (!dir.isDirectory()) {
                        String msg = "Included directory is not a directory: "+dir.getPath();
                        log(msg, org.apache.tools.ant.Project.MSG_WARN);
                    }
                    else {
                        vdirs.add(dir);
                    }
                }
            }

            // add dir names
            // check validity of these dirs
            NestedElementWithFile[] dirs = (NestedElementWithFile[])_includedir.toArray(new NestedElementWithFile[0]);
            for (int i=0;i<dirs.length;i++) {
                java.io.File dir = dirs[i].getFile();
                if (!dir.exists()) {
                    String msg = "Included directory does not exists: "+dir.getPath();
                    log(msg, org.apache.tools.ant.Project.MSG_WARN);
                }
                else if (!dir.isDirectory()) {
                    String msg = "Included directory is not a directory: "+dir.getPath();
                    log(msg, org.apache.tools.ant.Project.MSG_WARN);
                }
                else {
                    vdirs.add(dir);
                }
            }

            java.io.File[] idirs = (java.io.File[])vdirs.toArray(new java.io.File[0]);
            compileIDL(_idlfile, tmpdir, idirs, _prefix);
        }

        // build jar
        if (!jarUpToDate(tmpdir)) {
            // use BuildJarTask for that
            log("[execute] jar is not uptodate");
            buildjar().setSrcdir(tmpdir);
            buildjar().setCompileto(_compileto);
            buildjar().setArchivename(_archive);
            buildjar().setBuildto(_buildto);
            buildjar().execute();
        }

        // move files
        FileHelper.moveFiles(tmpdir.listFiles(), _generateto);

        // delete dir
        FileHelper.deleteDir(tmpdir);
    }

    //
    //  abstract operations
    //

    abstract protected void
    compileIDL(java.io.File idlfile, java.io.File compileto,
               java.io.File[] includedirs, String prefix);
}
