// ====================================================================
// 
//   ECM: The Extensible Container Model
//   Copyright (C) 2004 THALES
//   Contact: openccm-ecm@objectweb.org
//
//   This library is free software; you can redistribute it and/or
//   modify it under the terms of the GNU Lesser General Public
//   License as published by the Free Software Foundation; either
//   version 2.1 of the License, or any later version.
//
//   This library is distributed in the hope that it will be useful,
//   but WITHOUT ANY WARRANTY; without even the implied warranty of
//   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
//   Lesser General Public License for more details.
//
//   You should have received a copy of the GNU Lesser General Public
//   License along with this library; if not, write to the Free Software
//   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
//   USA
//
//   Initial developer(s): Mathieu Vadet.
//   Initial Funding: IST COACH European project (IST-2001-34445)
//                    http://www.ist-coach.org 
//
// ====================================================================



package org.objectweb.ccm.runtime;

import org.objectweb.corba.runtime.*;

/**
 **  <p>.</p>
 **/
public class ServerActivatorImpl
extends org.omg.CORBA.LocalObject
implements Servant, org.omg.Components.Deployment.ServerActivatorOperations
{
    // server activator
    static private String _class_name = "ServerActivatorImpl";
    static final private String _type_id =  org.omg.Components.Deployment.ServerActivatorHelper.id();
    private ORBService _orb_service;
    private org.omg.CosNaming.NamingContext _local_ns;
    private java.util.ArrayList _cs_refs;  // (list of ::Components::Deployment::ComponentServer)
    private int _cs_count;

    // default constructor
    public
    ServerActivatorImpl()
    {
        // server activator
        _orb_service = null;
        _local_ns = null;
        _cs_refs = new java.util.ArrayList();
        _cs_count = 0;
    }

    //
    //  Entry point
    //

    static public Servant
    create_servant()
    {
        return new ServerActivatorImpl();
    }

    //
    //  internal operations
    //

    //
    //  IDL:org/objectweb/corba/runtime/SystemComponent:1.0
    //

    final public void
    system_configuration_complete(SystemConfiguration cfg)
    {
        // obtain services
        ServantConfiguration scfg = (ServantConfiguration)cfg;
        _orb_service = scfg.orb_service();

        // obtain ref of local ns (should be registered with LocalNameService name)
        org.omg.CORBA.Object obj = null;
        obj = _orb_service.resolve_initial_references("LocalNameService");

        // check if found
        if (obj==null) {
            // NOTE: should not happen
            final String opname = "system_configuration_complete";
            final String msg = "FAILED (local NS not found)";
            TheLogger.error(_class_name, opname, msg);
            return ;
        }

        _local_ns = org.omg.CosNaming.NamingContextHelper.narrow(obj);

        // register ConfigValue factory
        String typeid = org.omg.Components.ConfigValueHelper.id();
        org.omg.CORBA.portable.ValueFactory fact = ConfigValueFactoryImpl.create_factory();
        _orb_service.register_valuefactory(new ValueFactoryImpl(typeid, fact));
    }

    final public void
    destroy()
    {
        // destroy all created CS
        org.omg.Components.Deployment.ComponentServer[] refs = get_component_servers();
        for (int i=0;i<refs.length;i++) {
            try {
                refs[i].remove();
            } catch (org.omg.Components.RemoveFailure ex) {
                // ignore
                final String opname = "destroy";
                TheLogger.debug(_class_name, opname, "IGNORE", ex);
                return ;
            }
        }

        // clear list
        _cs_refs.clear();

        // NOTE: TODO: should stop the process
    }

    //
    //  IDL:org/objectweb/corba/runtime/Servant:1.0
    //

    final public String
    type_id()
    {
        return _type_id;
    }

    final public org.omg.PortableServer.Servant
    as_native_servant()
    {
        return new org.omg.Components.Deployment.ServerActivatorPOATie(this);
    }

    //
    //  IDL:omg.org/Components/Deployment/ServerActivator:1.0
    //

    final public org.omg.Components.Deployment.ComponentServer
    create_component_server(org.omg.Components.ConfigValue[] config)
    throws org.omg.Components.CreateFailure,
           org.omg.Components.Deployment.InvalidConfiguration
    {
        // NOTE: TODO: rework this code
        //  - launch a thread which reads the subprocesses stdout/stderr to avoid deadlocks (see java doc)
        //  - use the cs_start scripts directly rather than the launcher

        final String opname = "create_component_server";
        String msg = null;
        // NOTE: start a new process
        String optionstr = System.getProperty("os.option.str");
        String libdir = System.getProperty("ECM.install.lib.dir");
        String xmldir = System.getProperty("ECM.install.launcher.dir");

        // use fake insname
        String insname = "ComponentServer"+_cs_count++;
        String targetxml = xmldir+java.io.File.separator+"corba_runtime.xml";

        // build args
        String ns_ior = _orb_service.object_to_string(_local_ns);
        org.omg.CORBA.Object obj = _orb_service.resolve_initial_references("ExtComponentInstallation");
        String extci_ior = _orb_service.object_to_string(obj);

        String[] args = {
            "-Darchive.location="+libdir+java.io.File.separator+"ccm_runtime_cs.jar",
            "-Darchive.entrypoint=org.objectweb.ccm.runtime.ComponentServerImpl.create_servant",
            "-Druntime.id="+insname,
            "-ArchiveLocation "+libdir+java.io.File.separator+"ccm_runtime_cs.jar",
            "-ArchiveEntrypoint org.objectweb.ccm.runtime.ComponentServerImpl.create_servant",
            "-RuntimeId "+insname,
            "-ORBInitRef NameService="+ns_ior,
            "-ORBInitRef ExtComponentInstallation="+extci_ior
        };

        // start a launcher
        org.objectweb.util.launcher.Launcher launcher = null;
        launcher = new org.objectweb.util.launcher.Launcher(targetxml, "corba_runtime", args);

        // get process to wait for message on stderr
        msg = "launcher started, wait on stderr/stdout";
        TheLogger.debug(_class_name, opname, msg);
        java.lang.Process csprocess = launcher.getProcess();
        java.io.InputStream err = csprocess.getErrorStream();
        java.io.InputStream out = csprocess.getInputStream();
        java.io.BufferedInputStream berr = new java.io.BufferedInputStream(err);
        java.io.BufferedInputStream bout = new java.io.BufferedInputStream(out);
        try {
            int errnum = 0;
            int outnum = 0;
            while (((errnum=berr.available())<=0) &&
                   ((outnum=bout.available())<=0)) {
                // if no byte available, then CS is not fully started, so wait 500 ms
                Thread.currentThread().sleep(500);
            }

            // read and log 
            if (errnum>0) {
                byte[] line = new byte[errnum];
                berr.read(line, 0, errnum);

                msg = "(STDERR)"+new String(line);
                TheLogger.debug(_class_name, opname, msg);
            }

            if (outnum>0) {
                byte[] line = new byte[outnum];
                bout.read(line, 0, outnum);

                msg = "(STDOUT)"+new String(line);
                TheLogger.debug(_class_name, opname, msg);
            }

            // start a reader thread to avoid deadlock on subprocess outputs (see javadoc)
            new SAConsoleReader(berr,bout).start();
        } catch (java.lang.Exception ex) {
            // ignore
            TheLogger.debug(_class_name, opname, "IGNORE", ex);
        } 

        // obtain ref from local NS
        RegistrationService regs = org.objectweb.corba.runtime.Runtime.getRegistrationService();
        INSRegistrationScheme insscheme = (INSRegistrationScheme)regs.get_scheme(INSRegistrationScheme.SCHEME_ID);
        obj = insscheme.lookup_in_ns(insname, _local_ns);

        // check if found
        if (obj==null) {
            msg = "FAILED ("+insname+" not found in local NS)";
            TheLogger.error(_class_name, opname, msg);
        }

        org.omg.Components.Deployment.ComponentServer cs = org.omg.Components.Deployment.ComponentServerHelper.narrow(obj);

        // store
        _cs_refs.add(cs);

        
        return cs;
    }

    final public void
    remove_component_server(org.omg.Components.Deployment.ComponentServer server)
    throws org.omg.Components.RemoveFailure
    {
        // remove from storage
        org.omg.Components.Deployment.ComponentServer[] refs = get_component_servers();
        int i=0;
        for (;i<refs.length;i++) {
            if (server._is_equivalent(refs[i])) {
                _cs_refs.remove(i);
                // NOTE: only call remove on the server if it was found to prevent a removal
                // of a server which is not owned by this SA
                server.remove();

                // NOTE: TODO: should stop the process

                break;
            }
        }

        // if it wasn't found, report removal failure
        if (i==refs.length) {
            throw new org.omg.Components.RemoveFailure();
        }
    }

    final public org.omg.Components.Deployment.ComponentServer[]
    get_component_servers()
    {
        return (org.omg.Components.Deployment.ComponentServer[])_cs_refs.toArray(new org.omg.Components.Deployment.ComponentServer[0]);
    }
}

//
// 
//
class SAConsoleReader
extends java.lang.Thread
{
    // reader
    static private String _class_name = "SAConsoleReader";
    private java.io.BufferedInputStream _err;
    private java.io.BufferedInputStream _out;
    private boolean _running;

    // default constructor
    public
    SAConsoleReader(java.io.BufferedInputStream out,
                    java.io.BufferedInputStream err)
    {
        // reader
        _err = err;
        _out = out;
        _running = false;
    }

    //
    //  public operations
    //

    final public void
    stopReader()
    {
        _running = false;
    }

    //
    //  java.lang.Thread
    //

    final public void
    run()
    {
        final String opname = "run";
        _running = true;

        while (_running) {
            try {
                int errnum = 0;
                int outnum = 0;
                while (((errnum=berr.available())<=0) &&
                       ((outnum=bout.available())<=0)) {
                    // if no byte available, wait 100 ms
                    Thread.currentThread().sleep(100);
                }

                // read and log 
                if (errnum>0) {
                    byte[] line = new byte[errnum];
                    berr.read(line, 0, errnum);

                    msg = "(STDERR)"+new String(line);
                    TheLogger.debug(_class_name, opname, msg);
                }

                if (outnum>0) {
                    byte[] line = new byte[outnum];
                    bout.read(line, 0, outnum);

                    msg = "(STDOUT)"+new String(line);
                    TheLogger.debug(_class_name, opname, msg);
                }

            } catch (java.lang.Exception ex) {
                // ignore
                TheLogger.debug(_class_name, opname, "IGNORE", ex);
            } 
        }
    }
}
