// ====================================================================
// 
//   ECM: The Extensible Container Model
//   Copyright (C) 2004 THALES
//   Contact: openccm-ecm@objectweb.org
//
//   This library is free software; you can redistribute it and/or
//   modify it under the terms of the GNU Lesser General Public
//   License as published by the Free Software Foundation; either
//   version 2.1 of the License, or any later version.
//
//   This library is distributed in the hope that it will be useful,
//   but WITHOUT ANY WARRANTY; without even the implied warranty of
//   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
//   Lesser General Public License for more details.
//
//   You should have received a copy of the GNU Lesser General Public
//   License along with this library; if not, write to the Free Software
//   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
//   USA
//
//   Initial developer(s): Mathieu Vadet.
//   Initial Funding: IST COACH European project (IST-2001-34445)
//                    http://www.ist-coach.org 
//
// ====================================================================



package org.objectweb.ccm.runtime;

import org.objectweb.corba.runtime.*;

/**
 **  <p>Segment specific implementation for the <tt>Components::Session2Context</tt> interface. This
 **  class implements the features which are specific to component instances in a particular
 **  home instance and is therefore provided to the component instances.</p>
 **/
public class SegmentedSession2ContextImpl
extends org.omg.CORBA.LocalObject
implements org.omg.Components.SegmentedSession2Context, org.coach.ECM.ECMContext
{
    // segmented session context
    static private String _class_name = "SegSegmentedSession2ContextImpl";
    private HomeSegmentedSession2ContextImpl _delegate;
    private org.omg.Components.LocalEvents _events;
    private org.omg.Components.LocalReceptacles _receptacles;

    // empty constructor
    protected
    SegmentedSession2ContextImpl()
    {
        // segmented session context
        _delegate = null;
        _events = null;
        _receptacles = null;
    }

    // default constructor
    protected
    SegmentedSession2ContextImpl(HomeSegmentedSession2ContextImpl del, byte[] mainsegid)
    {
        // segmented session context
        _delegate = del;
        _events = new LocalEventsImpl(del.getLocalEvents(), mainsegid);
        _receptacles = new LocalReceptaclesImpl(del.getLocalReceptacles(), mainsegid);
    }

    //
    //  IDL:omg.org/Components/CCMContext:1.0
    //

    final public org.omg.Components.CCMHome
    get_CCM_home()
    {
        return _delegate.get_CCM_home();
    }

    //
    //  IDL:omg.org/Components/CCM2Context:1.0
    //

    final public org.omg.Components.HomeRegistrationLocal
    get_home_registration()
    {
        return _delegate.get_home_registration();
    }

    final public void
    req_passivate()
    throws org.omg.Components.IllegalState,
           org.omg.Components.PolicyMismatch
    {
        _delegate.req_passivate();
    }

    final public org.omg.Components.LocalEvents
    get_events()
    {
        return _events;
    }

    final public org.omg.Components.LocalReceptacles
    get_receptacles()
    {
        return _receptacles;
    }

    final public void
    remove_component(org.omg.Components.CCMObject comp)
    {
        _delegate.remove_component(comp);
    }

    final public void
    register_valuefactory(String repid, org.omg.CORBA.portable.ValueFactory vfact)
    {
        _delegate.register_valuefactory(repid, vfact);
    }

    final public org.omg.CORBA.Any
    create_any()
    {
        return _delegate.create_any();
    }

    //
    //  IDL:omg.org/Components/SessionContext:1.0
    //

    final public org.omg.CORBA.Object
    get_CCM_object()
    throws org.omg.Components.IllegalState
    {
        return _delegate.get_CCM_object();
    }

    //
    //  IDL:omg.org/Components/SegmentedSession2Context:1.0
    //

    final public org.omg.Components.ComponentId
    get_component_id()
    throws org.omg.Components.IllegalState
    {
        return _delegate.get_component_id();
    }

    final public org.omg.Components.ComponentId
    create_main_segment(org.omg.Components.SegmentDescr[] descrs)
    {
        return _delegate.create_main_segment(descrs);
    }

    final public org.omg.CORBA.Object
    create_ref_from_cid(String repid,
                        org.omg.Components.ComponentId cid)
    {
        return _delegate.create_ref_from_cid(repid, cid);
    }

    final public org.omg.Components.ComponentId
    get_cid_from_ref(org.omg.CORBA.Object ref)
    throws org.omg.Components.BadComponentReference
    {
        return _delegate.get_cid_from_ref(ref);
    }

    //
    //  IDL:coach.org/ECM/ECMContext:1.0
    //

    final public org.omg.CORBA.Object
    get_service_internal(String sid, String siid)
    throws org.coach.ECM.UnknownService,
           org.coach.ECM.UnknownServiceInternal
    {
        return _delegate.get_service_internal(sid, siid);
    }
}
