// ====================================================================
// 
//   ECM: The Extensible Container Model
//   Copyright (C) 2004 THALES
//   Contact: openccm-ecm@objectweb.org
//
//   This library is free software; you can redistribute it and/or
//   modify it under the terms of the GNU Lesser General Public
//   License as published by the Free Software Foundation; either
//   version 2.1 of the License, or any later version.
//
//   This library is distributed in the hope that it will be useful,
//   but WITHOUT ANY WARRANTY; without even the implied warranty of
//   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
//   Lesser General Public License for more details.
//
//   You should have received a copy of the GNU Lesser General Public
//   License along with this library; if not, write to the Free Software
//   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
//   USA
//
//   Initial developer(s): Mathieu Vadet.
//   Initial Funding: IST COACH European project (IST-2001-34445)
//                    http://www.ist-coach.org 
//
// ====================================================================



package org.objectweb.ccm.logservice;

import org.objectweb.corba.runtime.*;
import org.objectweb.corba.logservice.*;

/**
 **  <p>Implementation of the <tt>ECA::ClientCallController</tt> and the
 **  <tt>ECA::ServerCallController</tt> OMG IDL interfaces.</p>
 **/
public class LogCallControllerImpl
extends org.omg.CORBA.LocalObject
implements org.coach.ECA.ClientCallController,
           org.coach.ECA.ServerCallController
{
    //
    private LogCurrent _current;

    public
    LogCallControllerImpl(LogCurrent current,
                          String scheme)
    {
        _current = current;
        // NOTE: scheme is not used currently
    }

    //
    //  internal operations
    //

    private LogCurrent
    getLogCurrent()
    {
        return _current;
    }

    //
    //  IDL:coach.org/ECA/CallController:1.0
    //

    final public void
    destroy()
    {
    }

    //
    //  IDL:coach.org/ECA/ClientCallController:1.0
    //

    final public void
    send_request(org.coach.ECA.ClientCallContext ctx)
    {
        // obtain log callback
        LogCallback callback = null;
        String cid = "";
        try {
            callback = (LogCallback)ctx.get_service_callback(LOG_SERVICE_ID.value, LOG_SERVICE_CALLBACK_ID.value);
            // obtain client id
            cid = callback.get_identity();
        } catch(org.coach.ECM.UnknownService ex) {
        } catch(org.coach.ECM.UnknownServiceCallback ex) {
        }

        // set client id in current
        org.omg.CORBA.Any acid = ctx.create_any();
        acid.insert_string(cid);
        getLogCurrent().current_id(acid);

        // build message
        MessageContent content = new MessageContent();
        java.util.Calendar cal = java.util.Calendar.getInstance();
        content.timestamp = java.lang.Long.toString(cal.getTimeInMillis());
        content.interception_point = "SEND_REQUEST";
        content.operation_name = ctx.call_information().operation_name;
        content.sender_id = cid;
        content.receiver_id = "";
        content.data = "";

        // obtain log internal
        LogInternal internal = null;        
        internal = (LogInternal)ctx.get_service_internal(LOG_SERVICE_ID.value, LOG_SERVICE_INTERNAL_ID.value);

        // send message
        LogEvent evt = new LogEvent();
        evt.level = INFO_LEVEL.value;
        evt.message = content;
        internal.log_event(evt);
    }

    final public void
    receive_reply(org.coach.ECA.ClientCallContext ctx)
    {
    }

    final public void
    receive_system_exception(org.coach.ECA.ClientCallContext ctx)
    {
    }

    final public void
    receive_user_exception(org.coach.ECA.ClientCallContext ctx)
    {
    }

    //
    //  IDL:coach.org/ECA/ServerCallController:1.0
    //

    final public void
    receive_request(org.coach.ECA.ServerCallContext ctx)
    {
        // get client id in current
        org.omg.CORBA.Any acid = getLogCurrent().current_id();
        String cid = "";
        if ((acid!=null) && (acid.type().kind()!=org.omg.CORBA.TCKind.tk_null)) {
            cid = acid.extract_string();
        }

        // obtain log callback
        LogCallback callback = null;
        String sid = "";
        try {
            callback = (LogCallback)ctx.get_service_callback(LOG_SERVICE_ID.value, LOG_SERVICE_CALLBACK_ID.value);
            // obtain client id
            sid = callback.get_identity();
        } catch(org.coach.ECM.UnknownService ex) {
        } catch(org.coach.ECM.UnknownServiceCallback ex) {
        }

        // build message
        MessageContent content = new MessageContent();
        java.util.Calendar cal = java.util.Calendar.getInstance();
        content.timestamp = java.lang.Long.toString(cal.getTimeInMillis());
        content.interception_point = "RECEIVE_REQUEST";
        content.operation_name = ctx.call_information().operation_name;
        content.sender_id = cid;
        content.receiver_id = sid;
        content.data = "";

        // obtain log internal
        LogInternal internal = null;        
        internal = (LogInternal)ctx.get_service_internal(LOG_SERVICE_ID.value, LOG_SERVICE_INTERNAL_ID.value);

        // send message
        LogEvent evt = new LogEvent();
        evt.level = INFO_LEVEL.value;
        evt.message = content;
        internal.log_event(evt);
    }

    final public void
    send_reply(org.coach.ECA.ServerCallContext ctx)
    {
    }

    final public void
    send_system_exception(org.coach.ECA.ServerCallContext ctx)
    {
    }

    final public void
    send_user_exception(org.coach.ECA.ServerCallContext ctx)
    {
    }
}
