// ====================================================================
// 
//   ECM: The Extensible Container Model
//   Copyright (C) 2004 THALES
//   Contact: openccm-ecm@objectweb.org
//
//   This library is free software; you can redistribute it and/or
//   modify it under the terms of the GNU Lesser General Public
//   License as published by the Free Software Foundation; either
//   version 2.1 of the License, or any later version.
//
//   This library is distributed in the hope that it will be useful,
//   but WITHOUT ANY WARRANTY; without even the implied warranty of
//   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
//   Lesser General Public License for more details.
//
//   You should have received a copy of the GNU Lesser General Public
//   License along with this library; if not, write to the Free Software
//   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
//   USA
//
//   Initial developer(s): Mathieu Vadet.
//   Initial Funding: IST COACH European project (IST-2001-34445)
//                    http://www.ist-coach.org 
//
// ====================================================================



package org.objectweb.corba.logservice;

import org.objectweb.corba.runtime.*;

/**
 **  <p>Implementation of the <tt>CORBA::Current</tt> interface for the log service.
 **  This current holds an 'id' which is propogated from the client-side to
 **  the server side by the request interceptors.</p>
 **
 **  @see org.objectweb.corba.logservice.LogSRIImpl LogSRIImpl
 **  @see org.objectweb.corba.logservice.LogCRIImpl LogCRIImpl
 **/
public class LogCurrentImpl
extends org.omg.CORBA.LocalObject
implements LogCurrent
{
    //
    private int _slot_id;
    private String _orb_id;

    public
    LogCurrentImpl(int slotid,
                   String orbid)
    {
        _slot_id = slotid;
        _orb_id = orbid;
     }

    //
    // internal operations
    //

    private org.omg.PortableInterceptor.Current
    getPICurrent()
    {
        org.omg.CORBA.Object obj = null;
        org.omg.PortableInterceptor.Current current = null;
        org.omg.CORBA.ORB orb = org.omg.CORBA.ORB.init();
        try {
            obj = orb.resolve_initial_references("PICurrent");
        } catch (org.omg.CORBA.ORBPackage.InvalidName ex) {
        } catch (Exception ex) {
            System.err.println("!!! LogCurrentImpl::getPICurrent exception caught: "+ex.getMessage());
        } catch (Error err) {
            System.err.println("!!! LogCurrentImpl::getPICurrent error caught: "+err.getMessage());
        }

        return org.omg.PortableInterceptor.CurrentHelper.narrow(obj);
    }

    private org.omg.CORBA.Any
    extractId()
    {
        // get id from slot
        System.err.println("### LogCurrentImpl::extractId called");
        try {
            return getPICurrent().get_slot(_slot_id);
        } catch (org.omg.PortableInterceptor.InvalidSlot ex) {
            System.err.println("!!! LogCurrentImpl::extractId slot_id does not exists: "+_slot_id);
        }

        return null;
    }

    private void
    insertId(org.omg.CORBA.Any id)
    {
        // set id in slot
        System.err.println("### LogCurrentImpl::insertId called");
        try {
            getPICurrent().set_slot(_slot_id, id);
        } catch (org.omg.PortableInterceptor.InvalidSlot ex) {
            System.err.println("!!! LogCurrentImpl::insertId slot_id does not exists: "+_slot_id);
        }
    }

    //
    //  IDL:org/objectweb/corba/logservice/LogCurrent:1.0
    //

    final public org.omg.CORBA.Any
    current_id()
    {
        return extractId();
    }

    final public void
    current_id(org.omg.CORBA.Any cid)
    {
        insertId(cid);
    }
}
